#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "actids.h"

#define MAX_OBJS            500

int NewObjType = -1;
int TotalLoaded = 0;
char buffer[1024];
int CustomBaseID = 200;

class ObjInfo
{
public:
    char *comment1;
    char *comment2;
    char *Id_name;
    char *name;
    char *HandlerName;
    int ObjType;
    int DidHandler;
    int x1, y1, x2, y2;

    ObjInfo(void) {
        DidHandler = ObjType = 0;
        comment1 = comment2 = Id_name = name = HandlerName = 0;
    }
    ObjInfo(int s) {
        DidHandler = ObjType = 0;
        comment1 = new char[s];
        comment2 = new char[s];
        Id_name = new char[s];
        name = new char[s];
        HandlerName = new char[s];
        CClear();
    }
    void CClear(void);
    ~ObjInfo() {
        delete comment1;
        delete comment2;
        delete Id_name;
        delete name;
        delete HandlerName;
    }
    void CSetInfo( char *idname, char *handn, char *_Name = 0,
              char *cmt1 = 0, char *cmt2 = 0 )
    {
        comment1 = cmt1;
        comment2 = cmt2;
        Id_name = idname;
        name = _Name;
        HandlerName = handn;
    }
    ObjInfo& operator=( ObjInfo &y );

};

void ObjInfo::CClear(void)
{
    if ( comment1 ) comment1[0] = 0;
    if ( comment2 ) comment2[0] = 0;
    if ( Id_name ) Id_name[0] = 0;
    if ( name ) name[0] = 0;
    if ( HandlerName ) HandlerName[0] = 0;
}

ObjInfo& ObjInfo::operator=( ObjInfo &y )
{
    if ( this == &y )
        return *this;
    delete comment1;
    delete comment2;
    delete Id_name;
    delete name;
    delete HandlerName;
    comment1 = comment2 = Id_name = name = HandlerName = 0;
    int len;
    if ( y.comment1 && (len = strlen(y.comment1)) > 0) {
        comment1 = new char [len+1];
        strcpy( comment1, y.comment1 );
    }
    if ( y.comment2 && (len = strlen(y.comment2)) > 0 ) {
        comment2 = new char [len+1];
        strcpy( comment2, y.comment2 );
    }
    if ( y.Id_name && (len = strlen(y.Id_name)) > 0 ) {
        Id_name = new char [len + 1];
        strcpy( Id_name, y.Id_name );
    }
    if ( y.name && (len = strlen(y.name)) > 0 ) {
        name = new char [len + 1];
        strcpy( name, y.name );
    }
    if ( y.HandlerName && (len = strlen(y.HandlerName)) > 0 ) {
        HandlerName = new char[len + 1];
        strcpy( HandlerName, y.HandlerName );
    }
    return *this;
}

ObjInfo ObjSlots[MAX_OBJS];


void ScanPosition( char *b, int *x1, int *y1, int *x2, int *y2 )
{
    int val[4], i = 0;

    while( *b != 0 ) {
        if ( *b == '+' ) {
            sscanf( b+1, "%d", &val[i]);
            i ++;
        }
        b ++;
    }
    *x1 = val[0];
    *y1 = val[1] + 20;
    *x2 = val[2];
    *y2 = val[3] + 20;
}

void swap( int *a, int *b )
{
    int t;
    t = *a;
    *a = *b;
    *b = t;
}


unsigned int CreateObject( int x1, int y1, int x2, int y2, int id = 0 )
{
    int NewObjectID = -1;
    int i;

    if ( NewObjType == -1 )
        return 0;
    if ( x1 > x2 )
        swap( &x1, &x2 );
    if ( y1 > y2 )
        swap( &y1, &y2 );
    if ( id == 0 ) {
        for( i = 0; i < MAX_OBJS; i ++ ) {
            if (ObjSlots[i].ObjType == 0) {
                NewObjectID = CustomBaseID + i;
                break;
            }
        }
    } else if ( id > 0 && id <= MAX_OBJS) {
        i = id - 1;
        if ( ObjSlots[i].ObjType == 0 )
            NewObjectID = i + CustomBaseID;
    }
    if ( NewObjectID == -1 ) {
        return 0;
    }
    ObjSlots[i].x1 = x1;
    ObjSlots[i].y1 = y1;
    ObjSlots[i].x2 = x2;
    ObjSlots[i].y2 = y2;
    ObjSlots[i].ObjType = NewObjType;
    TotalLoaded ++;
    return NewObjectID;
}

int Load( char* filename )
{
    FILE *file;
    int x1, y1, x2, y2, len;
    int type, id, oldtype = NewObjType;
    ObjInfo tmpOI(256);

    file = fopen( filename, "r");
    if ( file == NULL ) {
        printf("open file %s for reading failed.\n", filename );
        return -1;
    }
    while( fgets( buffer, 1024, file ) ) {
        len = strlen( buffer );
        if ( len > 0 ) len --;
        else
            fprintf(stderr, "load: string too short.\n");
        buffer[len] = 0;
        switch( buffer[0] ) {
            case '$':
                sscanf( buffer+1, "%d", &CustomBaseID );
                break;
            case '.':
                if ( len < 3 ) {
                    fprintf( stderr, "load: string '%s' short!\n", buffer );
                    break;
                }
                switch( buffer[1]) {
                    case 'i':
                        strcpy( tmpOI.Id_name, buffer+2 );
                        break;
                    case 'h':
                        strcpy( tmpOI.HandlerName, buffer+2);
                        break;
                    case 'n':
                        strcpy( tmpOI.name, buffer+2 );
                        break;
                    default:
                        break;
                }
                break;
            case '!':
                if ( len < 4 ) {
                    fprintf( stderr, "load: string '%s' short!\n", buffer );
                    break;
                }
                if ( buffer[1] == '1' ) {
                    strcpy( tmpOI.comment1, buffer+2 );
                } else {
                    strcpy( tmpOI.comment2, buffer+2 );
                }
                break;
            case ':':
                sscanf( buffer+1, "%d", &type );
                NewObjType = type;
                break;
            case '@':
                ScanPosition( buffer, &x1, &y1, &x2, &y2 );
                break;
            case '#':
                sscanf( buffer+1, "%d", &id );
                break;
            case '~':
                unsigned int NewID;
                NewID = CreateObject( x1, y1, x2, y2, id-(CustomBaseID - 1));
                if ( NewID >= CustomBaseID ) {
                    NewID -= CustomBaseID;
                    ObjSlots[NewID] = tmpOI;
                }
                tmpOI.CClear();
                break;
            default:
                break;
        }
    }
    NewObjType = oldtype;
    return 0;
}


void help(void)
{
    printf("usage: tosrc infile [-outfile] [.header] [x x_label x_value]");
    printf(" [y y_label y_value]\n");
    printf("       Will print to stdout if no outfile is specified.\n");
}

int main( int argc, char **argv)
{
    char OFFSX[] = "OFFX";
    char OFFSY[] = "OFFY";
    char *offsx = OFFSX;
    char *offsy = OFFSY;
    char *outname = 0;
    char *outhname = 0;
    int ox = 0, oy = 0;
    int argi = 2;

    if ( argc < 2 ) {
        help();
        exit(0);
    }

    if ((argc > argi && argv[argi][0] == '-')) {
       outname = argv[argi] + 1;
       argi ++;
    }

    outhname = outname;
    if ((argc > argi && argv[argi][0] == '.')) {
        outhname = argv[argi] + 1;
        argi ++;
    }

    for ( int i = 0; i < 2; i ++ ) {
        if ( argc >= argi+3 ) {
            if ( argv[argi][0] == 'x' ) {
                offsx = argv[argi+1];
                ox = atoi( argv[argi+2]);
            } else {
                offsy = argv[argi+1];
                oy = atoi( argv[argi+2]);
            }
            argi += 3;
        }
    }
    if ( argi != argc ) {
        printf("You probably made a mistake in the arguments.\n");
        help();
        exit(0);
    }


    printf( "// X OFFSET LABEL: %s\n", offsx );
    printf( "// X OFFSET VALUE: %d\n", ox );
    printf( "// Y OFFSET LABEL: %s\n", offsy );
    printf( "// Y OFFSET VALUE: %d\n", oy );

    if ( Load( argv[1]) ) {
        exit(0);
    }
    fflush( stdout );
    if ( outhname && freopen( outhname, "w", stdout ) == NULL ) {
       printf( "opening file %s failed.\n", outhname);
       exit(0);
    }

    if ( outhname ) {
        printf("#ifndef _");
        for ( i = 0; i < strlen( outhname); i ++ ) {
            if ( isalpha( outhname[i])) {
                printf("%c", toupper( outhname[i]));
            }
        }
        printf("_ID_\n");
        printf("#define _");
        for ( i = 0; i < strlen( outhname); i ++ ) {
            if ( isalpha( outhname[i])) {
                printf("%c", toupper( outhname[i]));
            }
        }
        printf("_ID_\n");
    }

    printf( "#define %s %d\n", offsx, ox );
    printf( "#define %s %d\n", offsy, oy );
    int id;
    printf( "#define %s_BASE_ID %5u\n", argv[1], CustomBaseID);
    printf( "#define %s_MAX_ID  %5u\n", argv[1], CustomBaseID+TotalLoaded);
    for ( i = 0; i < TotalLoaded; i ++ ) {
        if ( ObjSlots[i].ObjType ) {
            ObjInfo &t = ObjSlots[i];
            id = i + CustomBaseID;
            if (t.Id_name )
                printf("#define %20s %5u\n", t.Id_name, id );
            else {
                printf("// Error.  ID #%u has no name.\n", id );
                sprintf(buffer, "OBJECT_ID_%u", id );
                printf("#define %20s %5u\n", buffer, id );
                t.Id_name = new char[strlen(buffer)+1];
                strcpy( t.Id_name, buffer );
            }
        }
    }
    printf("\n#endif\n\n\n");
    fflush(stdout);
    fclose(stdout);
    if ( outname && freopen( outname, "w", stdout ) == NULL ) {
        printf( "opening file %s failed.\n", outname );
        exit(0);
    }
    if ( outhname )
        printf("#include \"%s\"\n", outhname );
    printf("\n#ifndef _I_HAVE_MY_OWN_HANDLERS_\n");
    for ( i = 0; i < TotalLoaded; i++ ) {
        if ( ObjSlots[i].ObjType && !ObjSlots[i].DidHandler ) {
            ObjInfo &t = ObjSlots[i];
            printf( "//\n");
            printf( "// Object ID Name: %s\n", t.Id_name );
            if ( t.comment1 )
                printf( "// %s\n", t.comment1 );
            if ( t.comment2 )
                printf( "// %s\n", t.comment2 );
            printf( "//\n");
            if ( t.HandlerName ) {
                for( int j = 0; j < TotalLoaded; j ++ ) {
                    if ( i != j && ObjSlots[j].ObjType &&
                         !ObjSlots[j].DidHandler &&
                         strcmp( ObjSlots[j].HandlerName,
                            t.HandlerName) == 0 ) {
                        printf( "// Object ID Name: %s\n",
                                ObjSlots[j].Id_name );
                        if ( ObjSlots[j].comment1 )
                            printf( "// %s\n", ObjSlots[j].comment1 );
                        if ( ObjSlots[j].comment2 )
                            printf( "// %s\n", ObjSlots[j].comment2 );
                        printf( "//\n");
                        ObjSlots[j].DidHandler = 1;
                    }
                }
            }
            if ( t.HandlerName ) {
                printf( "DeclareObjH( %s )\n{\n", t.HandlerName );
                if ( t.ObjType == BUTTON_ID )
                    printf("    if ( isClick( msg, LEFT_PRESSED)){}\n");
                printf( "    return 0;\n}\n\n" );
            } else {
                printf( "// Object has no handler name defined.\n\n");
            }
        }
    }
    printf( "#endif\n\nvoid %s_SetupObjects( ViewMgr &mgr ) \n{\n",
            argv[1]);
    for ( i = 0; i < TotalLoaded; i ++ ) {
        id = i + CustomBaseID ;
        if ( ObjSlots[i].ObjType ) {
            ObjInfo &t = ObjSlots[i];
            switch( t.ObjType ) {
                case SCROBJ_ID:
                    printf( "    mgr.CRegObject(\n%8snew ScrObj(\n%12s",
                            "", "");
                    printf( "%s + %d, %s + %d,\n%12s%s + %d, %s + %d,\n%12s",
                            offsx, t.x1, offsy, t.y1, "",
                            offsx, t.x2, offsy, t.y2,
                            "" );
                    if ( t.HandlerName )
                        printf( "%s, %s));\n\n", t.HandlerName, t.Id_name);
                    else
                        printf( " 0, %s));\n\n", t.Id_name );
                    break;
                case BUTTON_ID:
                    printf( "    mgr.CRegObject(\n%8snew Button(\n%12s",
                            "", "");
                    printf( "%s + %d, %s + %d,\n%12s%s + %d, %s + %d,\n%12s",
                            offsx, t.x1, offsy, t.y1, "",
                            offsx, t.x2, offsy, t.y2,
                            "" );
                    if ( t.HandlerName )
                        printf( "%s, %s", t.HandlerName, t.Id_name);
                    else
                        printf( " 0, %s", t.Id_name );
                    if ( t.name )
                        printf( ", \"%s\"", t.name );
                    printf(" ));\n\n");
                    break;
                case CHKBOX_ID:
                    printf( "    mgr.CRegObject(\n%8snew CheckBox(\n%12s",
                            "", "");
                    printf( "%s + %d, %s + %d,\n%12s%s + %d, %s + %d,\n%12s",
                            offsx, t.x1, offsy, t.y1, "",
                            offsx, t.x2, offsy, t.y2,
                            "" );
                    printf(" %s ));\n\n", t.Id_name );
                    break;
                case STRBOX_ID:
                    printf( "    mgr.CRegObject(\n%8snew StrBox(\n%12s",
                            "", "");
                    printf( "%s + %d, %s + %d,\n%12s%s + %d, %s + %d,\n%12s",
                            offsx, t.x1, offsy, t.y1, "",
                            offsx, t.x2, offsy, t.y2,
                            "" );
                    printf( "%s, ", t.Id_name );
                    if ( t.name )
                        printf(" string(\"%s\"), \n\n", t.name );
                    if ( t.HandlerName )
                        printf(" %s ));\n\n", t.HandlerName );
                    else
                        printf(" 0 ));\n\n" );
                    break;
                default:
                    printf("// Error. Unknown Object type id: %d\n",
                               t.ObjType );
                    break;
            }
        }
    }
    printf("}\n");
    if ( outname )
       fclose( stdout );
    return 0;
}
