//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// MODULE DIR                                                               //
//                                                                          //
//   This module implements a function to read all the file names in        //
// current directory.  The only callable function from this module is       //
// GetFiles.  Another function for the Class FileName is CSet.              //
// Please see dir.h for more info on Class FileName, which what this        //
// module use to hold filenames.                                            //
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <dir.h>
#include <string.h>
#include <dos.h>
#include "dir.h"

//
// Sets name and attrib of a filename.
//
void FileName::CSet( char *n, char unsigned a )
{
    strcpy( name, n );
    attrib = a;
}

int FileNameCmp( const void *a, const void *b )
{
    FileName &FNa = *((FileName*)a);
    FileName &FNb = *((FileName*)b);
    int d1 = FNa.CAtt() & 0x10;
    int d2 = FNb.CAtt() & 0x10;
    if ( d1 != d2 ) return d2 - d1;
    return ( strcmp( FNa.CName(), FNb.CName() ));
}

//
// GetFiles
//   The only important function from this module.
// wc = a string of the file mask to search for.  All subdirectories
//      will be searched automatically.
// fns = Array of FileName's to store the filenames.
//
// Returns: Total number of filenames found, including the subdirs.
//
int GetFiles( char *wc, FileName *fns )
{
    struct ffblk ffblk;
    int done, i = 0;
    char name[14];

    // Get subdirectories
    done = findfirst("*.*",&ffblk,_A_SUBDIR);
    while (!done) {
        if ( ffblk.ff_attrib & 0x10 ) {
            strcpy( name, ffblk.ff_name );
            strcat( name, "\\" );
            fns[i++].CSet( name, ffblk.ff_attrib );
        }
        done = findnext(&ffblk);
    }

    // Get files
    done = findfirst( wc, &ffblk, _A_NORMAL );
    while(!done) {
        fns[i++].CSet( ffblk.ff_name, ffblk.ff_attrib );
        done = findnext(&ffblk);
    }

    qsort( fns, i, sizeof( FileName ), FileNameCmp );

    return i;
}


// #define TEST
#ifdef TEST
main( int argc, char **argv )
{
    int t, i;
    FileName fns[512];
    if ( argc == 1 ) exit(0);
    t = GetFiles( argv[1], fns );
    for ( i = 0; i < t; i ++ ) {
        printf("%s\n", fns[i].CName());
    }
    printf("%d files.\n", t);
    return 0;
}
#endif
