#ifndef _OBJECT_H_
#define _OBJECT_H_
#include <stdio.h>
#include "classes.h"
#include "macros.h"
#include "depend.h"
#include "vidinter.h"
#include "mouinter.h"
#include "actids.h"

#ifdef __MSDOS__
#include <mem.h>
#endif

////////////////////////////////
//							  //
// Objects					  //
//							  //
////////////////////////////////

//
// Message manager & objects use to communicate with each other
//

// Allocate a big chunck of memory for cache.  Any pointer not fall into
// this chunck will be refer to default delete operator.  Otherwise use
// a private boolean member to remember if an message is from cache
// memory pool.  Right now address check only supports small model

// #define ADDRESS_CHECK_CACHE
#define MSG_CACHE_ON
#define MSG_CACHE_SIZE 100

class Message {
public:
    friend class ViewMgr;

    _pascal_ Message ( Atom *tobj, int id, Atom *caller, BYTE mb, int key )
        :ToObj( tobj ),     actId( id ),    Caller( caller ),
         mouButton( mb ),   keyDown( key ), next( 0 ),
#ifndef ADDRESS_CHECK_CACHE
         isCacheItem( cacheOn )
#endif
         {}

#ifdef MSG_CACHE_ON
    void* _pascal operator new(size_t s);
    void _pascal_ operator delete( void* pObj );
#endif  // MSG_CACHE_ON

	BYTE    mouButton;
    Atom *Caller, *ToObj;
	int     actId;
    int     keyDown;

protected:
	Message *next;

#ifdef MSG_CACHE_ON

    #ifndef ADDRESS_CHECK_CACHE
        BBOOL isCacheItem;
        static BBOOL cacheOn;
    #endif

        static Message* Cache[MSG_CACHE_SIZE];
        static int nCache;
        static char* CachePool;

#endif  // MSG_CACHE_ON
};

class Atom {
protected:
    Atom *next;
public:

    friend class AtomList;
    friend class ObjList;
    friend class ObjMgr;
    friend class ViewMgr;

    Atom():next(0){}
    ~Atom() {}
    virtual int _pascal_ identify(void) { return ATOM_ID; }
    Atom * _pascal_ CGetNext( void ) { return next; }
    void _pascal_ CSetNext( Atom *n ) { next = n; }
};

//
// Base of all objects
//

class Object:public Atom {
protected:
	WORD 	ObjID;
    OBJH    sHandler;
public:
	friend class ObjList;
    friend class ObjMgr;
	friend class ViewMgr;

     _pascal_ Object::Object( OBJH h=0, WORD id=0 )
        :ObjID(id),sHandler(h)          { }
    virtual  _pascal_ ~Object( void )           { }
    void     _pascal_ CSetID( WORD id )         { ObjID = id;}
    int      _pascal_ CGetID( void )            { return ObjID; }
    Object*  _pascal_ CNextObj( void )  { return (Object*)next; }
    void     _pascal_ CSetHandler( OBJH h )     { sHandler = h; }
    virtual int _pascal_ CSetDefaultHand( void ){ return 0; }
    virtual int _pascal_ identify(void)         { return OBJECT_ID; }
    Object&  _pascal_ operator=( const Object &y);
    virtual void _pascal_ CallBack( Message *msg ) {
            if ( sHandler ) {
               sHandler( msg );
            }
    }

}; // end of class Object


//
// Screen object (knows where itself is on the screen)
//

extern int objMouseX;
extern int objMouseY;

class ScrObj:public Object {

protected:
    static  ScrObj *HighLite;
	int 	sX1, sY1, sX2, sY2;

public:
     _pascal_ ScrObj(int x1, int y1, int x2, int y2, OBJH h = 0, WORD id = 0 );
     _pascal_ ScrObj( void ) { sX1 = sX2 = sY1 = sY2 = 0; }
    virtual       _pascal_ ~ScrObj();

    void          _pascal_ CLinkShowObject( void );
    ScrObj&       _pascal_ operator=( const ScrObj &y );
    virtual int   _pascal_ x1( void )           { return sX1; }
    virtual int   _pascal_ y1( void )           { return sY1; }
    virtual int   _pascal_ x2( void )           { return sX2; }
    virtual int   _pascal_ y2( void )           { return sY2; }
    virtual void  _pascal_ CShowObject( void );
    virtual int   _pascal_ identify( void )     { return SCROBJ_ID;   }
    virtual int   _pascal_ CRestoreOutLine( void );
    virtual int   _pascal_ CHighLiteOutLine( void );
    virtual void  _pascal_ CClrContext( int color = 0 );
    virtual void  _pascal_ CDrawOutLine( int color = OutLineColor );
    virtual void  _pascal_ CSetUpper (int x1 = 0, int y1 = 0 )
        { sX1 = x1; sY1 = y1; }
    virtual void  _pascal_ CSetLower (int x2 = 0, int y2 = 0 )
        { sX2 = x2; sY2 = y2; }
    virtual int   _pascal_ CMouseOnMe( void )
        { return MouseInRect( sX1-8, sY1-13, sX2, sY2 ); }
    virtual int   _pascal_ CMouseInMe( void )
        { return PointInRect( objMouseX, objMouseY, sX1, sY1, sX2, sY2 ); }

}; // end of class ScrObj

#endif
