//
// This module implements the palette editing dialog
// Side effects: It changes the global ScrObj Pointer MouButColor
//               to an object of its own.  However, it is restore once
//               the dialog is destroy.
//               It also creates a clone of the global object pointed to
//               by ColorChooserObj.
// Results:      Upon exit, any changes made to the palette table will stay
//               global.  Any changes to files will also be in effect as well.
//

#include <stdlib.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "viewmgr.h"   // View manager
#include "button.h"    // Push buttons
#include "actids.h"	   // Action ids
#include "dialog.h"
//#include "strbox.h"
#include "chkbox.h"
//#include "keys.h"
#include "sprdesg.h"
#include "buffer.h"
#include "filedlg.h"

#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>
#include <conio.h>
#endif

#ifndef BUFFER
#error Need a buffer of at least 768 bytes
#endif

#include "paldlg.id"
#include "paldlg.h"

BYTE rgb[3];
#define red rgb[0]
#define green rgb[1]
#define blue rgb[2]
static ScrObj *rgbScroll[3];
static ScrObj *rgbValueDisp[3];
static CheckBox *rgbLock[3];
static Button *rgbLeft[3];
static Button *rgbRight[3];
static ScrObj *CCO, *OldMBC; // Clone and saves the MouButColor pointer
static ScrObj *CurrentIndex;
static Button *PutPalObj, *SwapPalObj;
static WORD CurrentColorIndex;
static WORD PutIndex;
static BBOOL PutPalToTable = FALSE;
static WORD SwapIndex;
static BBOOL SwapPalToTable = FALSE;

int GetColorSlot( int ID )
{
    switch( ID ) {
        case RED_SCROLL_ID:
        case RED_LEFT_ID:
        case RED_RIGHT_ID:
        case RED_VALUE_DISP_ID:
            return 0;
        case GREEN_SCROLL_ID:
        case GREEN_LEFT_ID:
        case GREEN_RIGHT_ID:
        case GREEN_VALUE_DISP_ID:
            return 1;
        case BLUE_SCROLL_ID:
        case BLUE_LEFT_ID:
        case BLUE_RIGHT_ID:
        case BLUE_VALUE_DISP_ID:
            return 2;
        default:
            return -1;
    }
}

#define _I_HAVE_MY_OWN_HANDLERS_
//
// Object ID Name: BLUE_LEFT_ID
//
// Object ID Name: RED_LEFT_ID
//
// Object ID Name: GREEN_LEFT_ID
//
DeclareObjH( Left )
{
    Button &t = *(Button*)msg->ToObj;
    int i = GetColorSlot( t.CGetID());
    if ( msg->actId != -1 ) {
        if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
            if ( friction.peekTime() >= 1 )
                friction.start();
            else
                return 0;
        else if ( !(msg->mouButton & CLICK) )
            return 0;
        for ( int j = 0; j < 3; j ++ )
            if ( i != j && rgbLock[j]->CStatus())
                ViewMgr::CSendMsg( rgbLeft[j], -1, &t,
                                    msg->mouButton, msg->keyDown );
    }
    if (i > -1 ) {
        if ( rgb[i] > 0 ) {
            rgb[i] --;
            ViewMgr::CSendMsg(rgbScroll[i], REDRAW );
        }
        x_set_rgb( (BYTE)CurrentColorIndex, red, green, blue );
    }

    return 0;
}

//
// Object ID Name: RED_RIGHT_ID
//
// Object ID Name: BLUE_RIGHT_ID
//
// Object ID Name: GREEN_RIGHT_ID
//
DeclareObjH( Right )
{
    Button &t = *(Button*)msg->ToObj;

    int i = GetColorSlot( t.CGetID());
    if ( msg->actId != -1 ) {
        if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
            if ( friction.peekTime() >= 1 )
                friction.start();
            else
                return 0;
        else if ( !(msg->mouButton & CLICK) )
            return 0;
        for ( int j = 0; j < 3; j ++ )
            if ( i != j && rgbLock[j]->CStatus())
                ViewMgr::CSendMsg( rgbRight[j], -1, &t,
                                   msg->mouButton, msg->keyDown );
    }
    if (i > -1 ) {
        if ( rgb[i] < 63 ) {
            rgb[i] ++;
            ViewMgr::CSendMsg(rgbScroll[i], REDRAW );
        }
        x_set_rgb( (BYTE)CurrentColorIndex, red, green, blue );
    }
    return 0;
}

//
// Object ID Name: RED_SCROLL_ID
// Scroll red value
//
// Object ID Name: GREEN_SCROLL_ID
// Scroll green value
//
// Object ID Name: BLUE_SCROLL_ID
// blue value scroll
//
DeclareObjH( Scroll )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    int i = GetColorSlot(t.CGetID());
    static BBOOL MouScrolling = FALSE;
    switch( msg->actId ) {
        case REDRAW:
            int x = t.x1() + rgb[i] + 2, x1 = t.x1() + 1;
            int y1 = t.y1() + 1;
            m_HideMouse;
            v_RectFill( x, y1, t.x2(), t.y2(), 0 );
            v_RectFill( x1, y1, x, t.y2(), RGBColors[i] );
            ViewMgr::CSendMsg( rgbValueDisp[i], REDRAW );
            m_ShowMouse;
            break;
        case 0:
            break;
        default:
            return 0;

    }
    int X = objMouseX;
    int ColorValue;
    if ( msg->mouButton & HOLD ) {
        if ( !MouScrolling ) {
            MouScrolling = TRUE;
            ViewMgr::CSetHoldMsgCnt( FALSE );
            m_MouseWindow( t.x1()+1, t.y1()+1, t.x2()-1, t.y2()-1 );
        }
        ColorValue = X - (t.x1()+1);
        if ( ColorValue >= 0 && ColorValue <= 63 ) {
            if ( i > -1) {
                rgb[i] = ColorValue;
                ViewMgr::CSendMsg( &t, REDRAW );
                x_set_rgb( (BYTE)CurrentColorIndex, red, green, blue );
            }
        }
    } else if ( msg->mouButton & CLICK ) {
        ViewMgr::CSetHoldMsgCnt( TRUE );
        m_MouseWindow( 0, 0, v_Width -1, v_Height - 4 );
        MouScrolling = FALSE;
    }
    return 0;
}

//
// Object ID Name: CURRENT_PAL_ID
// Show current editing palette
//
DeclareObjH( CurrentPal )
{
    ScrObj &t = *((ScrObj*)msg->ToObj);
    switch( msg->actId ) {
        case MOU_BUT_CHANGE_COLOR:
            BYTE PutRGB[3];
            if ( PutPalToTable ) {
                x_get_pal_raw( PutRGB, 1, PutIndex );
                x_put_pal_raw( PutRGB, 1, tempLeftColor );
                PutPalToTable = FALSE;
                tempLeftColor = PutIndex;
                ViewMgr::CSendMsg( PutPalObj, -1 );
            } else if ( SwapPalToTable ) {
                BYTE SwapRGB[3];
                x_get_pal_raw( PutRGB, 1, SwapIndex );
                x_get_pal_raw( SwapRGB, 1, tempLeftColor );
                x_put_pal_raw( PutRGB, 1, tempLeftColor );
                x_put_pal_raw( SwapRGB, 1, SwapIndex );
                SwapPalToTable = FALSE;
                ViewMgr::CSendMsg( SwapPalObj, -1 );
            } else
                return 0;
            ViewMgr::CSendMsg( CCO, UNMARK_COLOR );
            break;
        case REDRAW:
            m_HideMouse;
            CurrentColorIndex = tempLeftColor;
            t.CClrContext( CurrentColorIndex );
            x_get_pal_raw( rgb, 1, CurrentColorIndex );
            for( int i = 0; i < 3; i ++ )
                ViewMgr::CSendMsg( rgbScroll[i], REDRAW );
            ViewMgr::CSendMsg( CurrentIndex, REDRAW );
            m_ShowMouse;
            break;
        default:
            break;
    }
    return 0;
}

//
// Object ID Name: SAVE_PAL_ID
// Save palette
//
DeclareObjH( SavePal )
{
    switch( msg->actId ) {
        case FILE_DLG_REPLY:
            x_get_pal_raw( (BYTE*)buffer, 256, 0 );
            writefile( ExportFileName, (char _FAR_*)buffer, 768 );
            return 0;
        default:
            break;
    }
    if ( isClick( msg, LEFT_PRESSED)) {
        GetFileName( "*.*", (Object*)msg->ToObj,
                     GET_FILE_NAME, "Save Palette" );
    }
    return 0;
}

//
// Object ID Name: LOAD_PAL_ID
// load palette
//
DeclareObjH( LoadPal )
{
    int LoadSize;
    switch( msg->actId ) {
        case FILE_DLG_REPLY:
            char _FAR_ *RawPal = readfile( ExportFileName, LoadSize );
            if ( LoadSize != 768 ) {
                sprintf( buffer, "LoadSize in LoadPal: %d.", LoadSize );
                v_PrintMsg( 0, 0, TextColor, FaceColor, buffer );
            }
            if ( RawPal ) {
                v_PutPal( (BYTE*)RawPal );
                delete RawPal;
            }
            return 0;
        default:
            break;
    }
    if ( isClick( msg, LEFT_PRESSED)) {
        GetFileName( "*.*", (Object*)msg->ToObj,
                     GET_FILE_NAME, "Load Palette" );
    }
    return 0;
}

//
// Object ID Name: PUT_PAL_ID
// put current palette into a color slot
DeclareObjH( PutPal )
{
    static char *name = 0;
    static char _FAR_ val[4];
    Button &t = *((Button*)msg->ToObj);
    if ( !name ) name = t.CName();
    if ( isClick( msg, LEFT_PRESSED)) {
        if ( !SwapPalToTable ) {
            PutIndex = CurrentColorIndex;
            PutPalToTable = !PutPalToTable;
            if ( PutPalToTable ) {
                m_HideMouse;
                sprintf( val, "%3u", PutIndex );
                t.CSetName( val );
                t.CClrContext( PutIndex );
                t.CShowName();
                m_ShowMouse;
                ViewMgr::CSendMsg( CCO, MARK_COLOR );
            } else {
                t.CSetName( name );
                t.CShowObject();
            }
        }
    } else if ( msg->actId == -1 ) {
        t.CSetName( name );
        t.CShowObject();
    }
    return 0;
}

//
// Object ID Name: SWAP_PAL_ID
//
DeclareObjH( SwapPal )
{
    static char *name = 0;
    static char _FAR_ val[5];
    Button &t = *((Button*)msg->ToObj);
    if ( !name ) name = t.CName();
    if ( isClick( msg, LEFT_PRESSED )) {
        if ( !PutPalToTable ) {
            SwapIndex = CurrentColorIndex;
            SwapPalToTable = !SwapPalToTable;
            if ( SwapPalToTable ) {
                m_HideMouse;
                sprintf( val, "%3u", SwapIndex );
                t.CSetName( val );
                t.CClrContext( SwapIndex );
                t.CShowName();
                m_ShowMouse;
                ViewMgr::CSendMsg( CCO, MARK_COLOR );
            } else {
                t.CSetName( name );
                t.CShowObject();
            }
        }
    } else if ( msg->actId == -1 ) {
        t.CSetName( name );
        t.CShowObject();
    }
    return 0;
}

//
// Object ID Name: GREEN_VALUE_DISP_ID
// display Green value
//
// Object ID Name: BLUE_VALUE_DISP_ID
// display Blue value
//
// Object ID Name: RED_VALUE_DISP_ID
// display Red value
//
DeclareObjH( RGBValue )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    if ( msg->actId == REDRAW ) {
        int i = GetColorSlot( t.CGetID());
        sprintf( buffer, "%2u", rgb[i]);
        v_PrintMsg( v_CenterMsgX( t.x1(), t.x2(), buffer ),
                    v_CenterMsgY(  t.y1(), t.y2()),
                    TextColor, FaceColor, buffer );
        m_ShowMouse;
    }
    return 0;
}

//
// Object ID Name: CURREN_COLOR_NUM_ID
// display current color's index number
//
DeclareObjH( CurrentColorNum )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    if ( msg->actId == REDRAW ) {
        sprintf(buffer, "%3u", CurrentColorIndex );
        v_PrintMsg( v_CenterMsgX( t.x1(), t.x2(), buffer ),
                    v_CenterMsgY(t.y1(), t.y2()),
                    TextColor, FaceColor, buffer );
    }
    return 0;
}


#include "paldlg.src"


void SetLocalObjectPointers( Dialog *dlg )
{
    rgbScroll[0] = (ScrObj*)dlg->CGetObject(RED_SCROLL_ID);
    rgbScroll[1] = (ScrObj*)dlg->CGetObject(GREEN_SCROLL_ID);
    rgbScroll[2] = (ScrObj*)dlg->CGetObject(BLUE_SCROLL_ID);
    rgbValueDisp[0] = (ScrObj*)dlg->CGetObject(RED_VALUE_DISP_ID);
    rgbValueDisp[1] = (ScrObj*)dlg->CGetObject(GREEN_VALUE_DISP_ID);
    rgbValueDisp[2] = (ScrObj*)dlg->CGetObject(BLUE_VALUE_DISP_ID);
    rgbLock[0] = (CheckBox*)dlg->CGetObject(RED_LOCK_ID);
    rgbLock[1] = (CheckBox*)dlg->CGetObject(GREEN_LOCK_ID);
    rgbLock[2] = (CheckBox*)dlg->CGetObject(BLUE_LOCK_ID);
    rgbLeft[0] = (Button*)dlg->CGetObject(RED_LEFT_ID);
    rgbLeft[1] = (Button*)dlg->CGetObject(GREEN_LEFT_ID);
    rgbLeft[2] = (Button*)dlg->CGetObject(BLUE_LEFT_ID);
    rgbRight[0] = (Button*)dlg->CGetObject(RED_RIGHT_ID);
    rgbRight[1] = (Button*)dlg->CGetObject(GREEN_RIGHT_ID);
    rgbRight[2] = (Button*)dlg->CGetObject(BLUE_RIGHT_ID);
    CurrentIndex = (ScrObj*)dlg->CGetObject(CURRENT_COLOR_NUM_ID);
    PutPalObj = (Button*)dlg->CGetObject(PUT_PAL_ID);
    SwapPalObj =(Button*)dlg->CGetObject(SWAP_PAL_ID);
    // Redirect the global object pointer MouButColor
    OldMBC = MouButColor;
    MouButColor =(ScrObj*) dlg->CGetObject( CURRENT_PAL_ID );
    // Create a clone of object pointed to by ColorChooserObj
    CCO = new ScrObj( *ColorChooserObj );
}

DeclareObjH( EditPalette )
{
    static Dialog *dlg = 0;
    static installed = 0;;
    int i;

    switch( msg->actId ) {
        case -1:
            if ( dlg == 0 ) {
                dlg = new Dialog( 132, 68, 263, 197, 0, 126 );
                paldlg_SetupObjects(*dlg);
                SetLocalObjectPointers(dlg);
                ViewMgr::CSendMsg( dlg, INSTALL_DIALOG,
                                   dlg->CGetObject(OK_BUTTON_PAL_ID));
                friction.start();
                tempLeftColor = PutIndex = SwapIndex =
                    CurrentColorIndex = LeftColor;
                PutPalToTable = SwapPalToTable = FALSE;
            }
            return 0;
        case INSTALL_DIALOG:
            if ( !installed ) {
                sprintf( buffer, "Palette Editor");
                v_PrintMsg( v_CenterMsgX(dlg->x1(), dlg->x2(), buffer),
                            dlg->y1() + 5, TextColor, FaceColor, buffer );
                // Register CCO here so dialog doesn't send redraw to it
                // when initialize.
                dlg->CRegObject( CCO );
                installed = 1;
            }
            return 0;
        case DESTROY_DIALOG:
            if ( dlg ) {
                for( i = paldlg_BASE_ID; i < paldlg_MAX_ID; i ++ ) {
                    delete (dlg->CGetObject(i));
                }
                delete CCO;
                delete dlg;
            }
            installed = 0;
            MouButColor = OldMBC;  // Restore global pointer MouButColor
            dlg = 0;
            return 0;
        case 0:
        case MOUSE_OFF:
            break;
        default:
            return 0;
    }
    if ( dlg && isClick( msg, LEFT_PRESSED )) {
        ScrObj &t = *(ScrObj*)msg->ToObj;
        if ( t.CGetID() == OK_BUTTON_PAL_ID ) {
            ViewMgr::CSendMsg( dlg, DIALOG_CONFIRM, dlg->CGetObject(
                               OK_BUTTON_PAL_ID ));
        }
    }
    return 0;
}
