#include <stdlib.h>
#include <process.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Push buttons
#include "viewmgr.h"   // View manager
#include "actids.h"	   // Action ids
#include "dialog.h"
#include "strbox.h"
#include "chkbox.h"
#include "keys.h"
#include "buffer.h"
#include "savesize.h"
#include "savesize.id"
#include "mousecur.h"

#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>
#include <conio.h>
#endif

extern Timer friction;

char _FAR_ Dot[] = { 1, 0,0,0,0,0,0,0,0,0,0,0,0,0 };

static Dialog *dlg = 0;
static Object *reply = 0;
static ScrObj *WidthValue;
static ScrObj *HeightValue;
static ScrObj *SSCanvas;
static ScrObj *SSViewer;

static int SSMaxW, SSMaxH;
static int *X1 = 0;
static int *Y1 = 0;
static int *SSWidth = 0;
static int *SSHeight = 0;
static int ox1, oy1;  // Use by SSCanvas
static int vox1 = -1, voy1, vox2, voy2;  // use by SSViewer
static char _FAR_ *SS_SprData;
static char _FAR_ *SS_RecData;
static int MouButDown = 0;  // Indicate if using mouse to move box
static int InitSS = 0; // Indicate if RecData has been initialize
static char DefTitle[] = "Select Area for Sprite.";
static char *titleStr = DefTitle;
static int ReplyId;

#define _I_HAVE_MY_OWN_HANDLERS_

//
// Object ID Name: LEFT_ID
//
// Object ID Name: UP_ID
//
// Object ID Name: RIGHT_ID
//
// Object ID Name: DOWN_ID
//
DeclareObjH( MoveBox )
{
    if ( !SSCanvas )
        return 0;
    Button &t = *(Button*)msg->ToObj;
    if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
        if ( friction.peekTime() >= 1 )
            friction.start();
        else
            return 0;
    else if ( !(msg->mouButton & CLICK) )
        return 0;

    switch( t.CGetID()) {
        case LEFT_ID:
            // 1 is reserve for canvas' outline
            if ( (*X1) > 1 ) (*X1) --;
            else    return 0;
            break;
        case UP_ID:
            if ( (*Y1) > 1 ) (*Y1) --;
            else    return 0;
            break;
        case RIGHT_ID:
            if ( (*X1) <= (SSMaxW-*SSWidth) ) (*X1) ++;
            else    return 0;
            break;
        case DOWN_ID:
            if ( (*Y1) <= (SSMaxH-*SSHeight) ) (*Y1) ++;
            else    return 0;
            break;
        default:
            break;
    }
    dlg->CSendMsg( SSCanvas, REDRAW );

    return 0;
}

//
// Object ID Name: MOUSE_MOV_ID
//
// Object ID Name: WID_VALUE_ID
//
// Object ID Name: HEI_VALUE_ID
//
// Object ID Name: WIDTH_LABEL_ID
//
// Object ID Name: HEIGHT_LABEL_ID
//
// Object ID Name: TITLE_ID
//
DeclareObjH( Labels )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    if ( msg->actId == REDRAW ) {
        switch( t.CGetID() ) {
            case WIDTH_LABEL_ID:
                sprintf( buffer, "W");
                break;
            case HEIGHT_LABEL_ID:
                sprintf( buffer, "H");
                break;
            case TITLE_ID:
                sprintf( buffer, "%s", titleStr );
                break;
            case WID_VALUE_ID:
                sprintf( buffer, "%3u", (*SSWidth) );
                break;
            case HEI_VALUE_ID:
                sprintf( buffer, "%3u", (*SSHeight) );
                break;
            case MOUSE_MOV_ID:
                sprintf( buffer, "X1=%3u   Y1=%3u", *X1-1, *Y1-1 );
                break;
            default:
                return 0;
        }
        v_PrintMsg( v_CenterMsgX( t.x1(), t.x2(), buffer),
                    v_CenterMsgY( t.y1(), t.y2() ),
                    TextColor, FaceColor, buffer );
    }
    return 0;
}

//
// Object ID Name: DEC_WID_ID
//
// Object ID Name: DEC_HEI_ID
//
DeclareObjH( Decrease )
{
    Button &t = *(Button*)msg->ToObj;
    if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
        if ( friction.peekTime() >= 1 )
            friction.start();
        else
            return 0;
    else if ( !(msg->mouButton & CLICK) )
        return 0;

    if ( t.CGetID() == DEC_WID_ID ) {
        if ( (*SSWidth) > 4 ) (*SSWidth) -= 4;
        else return 0;
        dlg->CSendMsg( WidthValue, REDRAW );
    } else {
        if ( (*SSHeight) > 1 ) (*SSHeight) --;
        else return 0;
        dlg->CSendMsg( HeightValue, REDRAW );
    }
    dlg->CSendMsg( SSCanvas, REDRAW );
    return 0;
}

// Object ID Name: INC_WID_ID
//
// Object ID Name: INC_HEI_ID
//
DeclareObjH( Increase )
{
    Button &t = *(Button*)msg->ToObj;
    if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
        if ( friction.peekTime() >= 1 )
            friction.start();
        else
            return 0;
    else if ( !(msg->mouButton & CLICK) )
        return 0;

    if ( t.CGetID() == INC_WID_ID ) {
        if ( (*SSWidth+*X1) < SSMaxW ) (*SSWidth) += 4;
        else return 0;
        dlg->CSendMsg( WidthValue, REDRAW );
    } else {
        if ( (*SSHeight+*Y1) <= SSMaxH ) (*SSHeight) ++;
        else return 0;
        dlg->CSendMsg( HeightValue, REDRAW );
    }
    dlg->CSendMsg( SSCanvas, REDRAW );
    return 0;
}

DeclareObjH( SaveSizeDlg );


#include "savesize.src"

static int _pascal_ SetupDialog( void )
{
    if ( !dlg ) {
        dlg = new Dialog(
                    SSOFFX + 66, SSOFFY + 188,
                    SSOFFX + 233, SSOFFY + 260,
                     0, 100 );
        if ( !dlg ) {
            v_PrintMsg( 0, 0, TextColor, FaceColor, "New Failed! SSDlg");
            return 0;
        }
        savesize_SetupObjects(*dlg);
        dlg->CRegObject( SSCanvas );
        dlg->CRegObject( SSViewer );
        ViewMgr::CSendMsg( dlg, INSTALL_DIALOG, dlg->CGetObject(OK_ID));
        WidthValue = (ScrObj*)dlg->CGetObject( WID_VALUE_ID );
        HeightValue = (ScrObj*)dlg->CGetObject( HEI_VALUE_ID );
        ReplyId = CANCEL_GET_SIZE;
    }
    return 0;
}

static int _pascal_ DestroyDialog( void )
{
    if ( dlg ) {
        for( int i = savesize_BASE_ID; i < savesize_MAX_ID; i ++ )
            delete( dlg->CGetObject(i));
        delete dlg;
        if ( SSCanvas ) {
            delete SSCanvas;
            SSCanvas = 0;
        }
        if ( SSViewer ) {
            delete SSViewer;
            SSViewer = 0;
        }
        dlg = 0;
        ViewMgr::CSendMsg( reply, ReplyId );
    }
    return 0;
}

static void _pascal_ CheckDlgButtons( Message *msg )
{
    if ( dlg && isClick( msg, LEFT_PRESSED )) {
        Button &t = *(Button*)(msg->ToObj);
        switch( t.CGetID()) {
            case OK_ID:
                dlg->CSendMsg( dlg, DIALOG_CONFIRM, &t );
                break;
            case CANCEL_ID:
                dlg->CSendMsg( dlg, DIALOG_CANCEL, &t );
                break;
            default:
                break;
        }
    }
}

DeclareObjH( SaveSizeDlg )
{
    switch( msg->actId ) {
        case GET_SAVE_SIZE:
            return SetupDialog();
        case DESTROY_DIALOG:
            return DestroyDialog();
        case DIALOG_CONFIRM:
            ReplyId = CONFIRM_GET_SIZE;
            return 0;
        case DIALOG_CANCEL:
            ReplyId = CANCEL_GET_SIZE;
            return 0;
        default:
            break;
    }
    CheckDlgButtons( msg );
    return 0;
}

DeclareObjH( SSCanvasHandler )
{
    static ow, oh;

    ScrObj &t = *(ScrObj*)(msg->ToObj);

    int x1 = *X1 + t.x1();
    int y1 = *Y1 + t.y1();

    if ( msg->actId == REDRAW ) {
        m_HideMouse;
        if ( ox1 > 0 ) {
            v_Rectangle( ox1, oy1, ox1+ow-1, oy1+oh-1, v_cZero );
        } else {
            InitSS = 1;  // Now we can use the viewer
            t.CClrContext();
        }
        v_Putbmp( t.x1()+1, t.y1()+1, SS_SprData );
        v_Getbmp( x1, y1, *SSWidth, *SSHeight, SS_RecData );
        v_Rectangle( x1, y1, x1+(*SSWidth)-1, y1+(*SSHeight)-1, v_cWhite );
        ox1 = x1, oy1 = y1, ow = *SSWidth, oh = *SSHeight;
        if ( !MouButDown ) m_ShowMouse;
        dlg->CSendMsg( dlg->CGetObject( MOUSE_MOV_ID), REDRAW );
        dlg->CSendMsg( SSViewer, REDRAW );
        return 0;   // Don't forget to return so won't execute the redraw
                    // send at the end of the function.  otherwise infinite
                    // message loop is created and once the message pool runs
                    // out, ka-poola-boom... crash!!!!
    } else if ( msg->mouButton == LHOLD ) {
        if ( MouButDown ) {
            *X1 = objMouseX - t.x1();
            *Y1 = objMouseY - t.y1();
            if ( objMouseX == ox1 && objMouseY == oy1 )
                return 0;
        } else {
            MouButDown = 1;
            m_HideMouse;
            m_MouseWindow( t.x1()+1, t.y1()+1,
                           t.x1()+1+SSMaxW-*SSWidth,
                           t.y1()+1+SSMaxH-*SSHeight );
            m_MouseSetXY( x1, y1 );
            m_UpdateMouse;
            x_define_mouse_cursor( Dot, MouseCursorColor );
        }
    } else if ( msg->mouButton == LCLICK ) {
        m_MouseWindow( 0, 0, v_Width - 1, v_Height - 4 );
//andout	x_define_def_mouse_cursor( MouseCursorColor );
	x_define_mouse_cursor( defCursor, MouseCursorColor );
        m_ShowMouse;
        MouButDown = 0;
    } else
        return 0;
    dlg->CSendMsg( &t, REDRAW );
    return 0;
}

DeclareObjH( SSViewerHandler )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    if ( msg->actId == REDRAW && InitSS ) {
        int x1, y1, x2, y2;
        x1 = ( t.x2() - t.x1() - *SSWidth ) / 2 + t.x1();
        y1 = ( t.y2() - t.y1() - *SSHeight ) / 2 + t.y1();
        x2 = x1 + *SSWidth + 1;
        y2 = y1 + *SSHeight + 1;
        if ( !MouButDown )
            m_HideMouse;
        if ( vox1 != x1 || voy1 != y1 || vox2 != x2 || voy2 != y2 ) {
            t.CClrContext();
            t.CDrawOutLine();
            v_PutLine( x1, t.y1(), x1, t.y2(), v_cWhite );
            v_PutLine( x2, t.y1(), x2, t.y2(), v_cWhite );
            v_PutLine( t.x1(), y1, t.x2(), y1, v_cWhite );
            v_PutLine( t.x1(), y2, t.x2(), y2, v_cWhite );
            vox1 = x1, voy1 = y1, vox2 = x2, voy2 = y2;
        }
        v_Putbmp( x1+1, y1+1, SS_RecData );
        if ( !MouButDown )
            m_ShowMouse;
    }
    return 0;
}

static void _pascal_ Go( Object *Caller, int SendId, int *x1, int *y1,
                         int *w, int *h, int MaxW, int MaxH, char *Title )
{
    SSMaxW = MaxW;
    SSMaxH = MaxH;

    SSWidth = w;
    SSHeight = h;

    X1 = x1;
    Y1 = y1;

    if ( *SSWidth < 4 ) *SSWidth = 4;
    if ( *SSHeight < 1 ) *SSHeight = 1;

    if ( *SSWidth > MaxW ) *SSWidth = MaxW;
    if ( *SSHeight > MaxH ) *SSHeight = MaxH;

    if ( *X1+*SSWidth-1 > MaxW ) *X1 = MaxW - *SSWidth;
    if ( *Y1+*SSHeight-1 > MaxH ) *Y1 = MaxH - *SSHeight;

    if ( *X1 < 1 ) *X1 = 1;
    if ( *Y1 < 1 ) *Y1 = 1;

    ox1 = -1; // Reset SSCanvas
    vox1 = -1; // Reset SSViewer

    InitSS = 0;

    reply = (Object*)Caller;

    if ( Title )
        titleStr = Title;
    else
        titleStr = DefTitle;

    Message *msg = new Message( 0, SendId, Caller, 0, 0 );
    SaveSizeDlg( msg );
    delete msg;
}

void _pascal_ GetWH( Object *Caller, int SendId,
                     int *w, int *h, int MaxW, int MaxH, char *Title )
{
    static int myX, myY;

    myX = myY = 1;

    SSCanvas = 0;
    SSViewer = 0;
    SS_SprData = SS_RecData = 0;
    Go( Caller, SendId, &myX, &myY, w, h, MaxW, MaxH, Title );
}


void _pascal_ GetSaveSize( Object *Caller, int SendId,
                           ScrObj *Canvas, ScrObj *Viewer,
                           char _FAR_ *SprData, char _FAR_ *ReceiveData,
                           int *x1, int *y1, int *w, int *h,
                           int MaxW, int MaxH, char *Title )
{
    SS_SprData = SprData;
    SS_RecData = ReceiveData;

    SSCanvas = new ScrObj( *Canvas );
    SSCanvas->CSetHandler( SSCanvasHandler );
    SSCanvas->CSetID( savesize_MAX_ID );

    SSViewer = new ScrObj( *Viewer );
    SSViewer->CSetHandler( SSViewerHandler );
    SSViewer->CSetID( savesize_MAX_ID + 1 );

    Go( Caller, SendId, x1, y1, w, h, MaxW, MaxH, Title );
}

