#include <stdio.h>
#include <stdlib.h>
#include <dir.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include "macros.h"
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Implements Push buttons
#include "viewmgr.h"   // Implements View manager
#include "actids.h"	   // Action ids
#include "dialog.h"    // Simple Dialog Handler
#include "strbox.h"    // Implements the string box
#include "chkbox.h"    // Implements the Check Box
#include "keys.h"      // Key codes
#include "sprdesg.h"   // Header file for this module
#include "sprdesg.id"  // Object IDs for all the objects in this module
#include "filedlg.h"   // Header file for the File Dialog module
#include "buffer.h"    // Common buffer (1024 bytes)
#include "xbmtools.h"  // XLib bitmap tools
#include "scale1.h"
#include "errcode.h"
#include "sprviewr.h"
#include "spcanvas.h"

#include "smalview.h"
#include <conio.h>

zSmallView SmallView;

int FirstViewerIdx = 0;
ScrObj *CurrentViewer;

extern Timer friction;

_pascal_ zSmallView::~zSmallView()
{
}

void _pascal_ zSmallView::NewList( zSprite *temp )
{
    Count = CurrentIdx = 0;
    TrailPtr = 0;
    head = CurrentPtr = tail = temp;
}


zSprite * _pascal_ zSmallView::InsertBlankBefore( void )
{
    zSprite *temp = new zSprite;

    // Gotta be an empty list.
    if ( !head ) {
        NewList( temp );

    } else if ( CurrentPtr == head ) {  // Before head

        CurrentIdx = 0;
        temp->next = head;
        head = temp;

    } else if ( TrailPtr ) {  // In the middle
                                // (CurrentPtr can't point beyonded tail)
        TrailPtr->next = temp;
        temp->next = CurrentPtr;

    } else {
        delete temp;
        return 0;
    }

    Count ++;
    CurrentPtr = temp;
    return temp;
}

zSprite *_pascal_ zSmallView::InsertBlankAfter( void )
{
    zSprite *temp = new zSprite;

    // Gotta be an empty list.
    if ( !head ) {
        NewList( temp );

    } else if ( CurrentPtr ) {

        zSprite *next = CurrentPtr->next;  // Save next

        CurrentPtr->next = temp;           // Insert after current
        temp->next = next;              // Connect list

        if ( CurrentPtr == tail )       // Check for tail
            tail = temp;

    } else {
        delete temp;
        return 0;
    }
    Count ++;
    return temp;
}

zSprite * _pascal_ zSmallView::AddBlank( void )
{
    zSprite *temp = new zSprite;
    if ( !head ) {
        NewList( temp );
    } else {
        tail->next = temp;
        tail = temp;
    }
    Count ++;
    return temp;
}

int _pascal_ zSmallView::RemoveCurrent( void )
{
    if ( Count )
        Count --;
    else
        return 0;

    zSprite *temp = CurrentPtr;

    if ( !Count ) {                             // All gone
        NewList(0);                                 // Make an empty list

    } else if ( CurrentPtr == head ) {          // It's head

        // Don't have to worry about tail because empty list is handle above
        CurrentPtr = head = head->next;             // Remove head

    } else if ( CurrentPtr == tail ) {          // It's tail

        TrailPtr->next = 0;
        tail = TrailPtr;
        FindSprite( CurrentIdx-1, 1 );

    } else if ( CurrentPtr ) {                  // In the middle

        TrailPtr->next = CurrentPtr->next;
        FindSprite( TrailPtr->next, 1 );

    } else
        return 0;                               // What???!!!

    delete temp;
    return 1;
}

int _pascal_ zSmallView::PutSprite( char _FAR_ *data, char *name )
{
    if ( !head )
        InsertBlankBefore();
    if ( !CurrentPtr ) {
        CurrentIdx = 0;
        TrailPtr = 0;
        CurrentPtr = head;
    }
    delete CurrentPtr->data;
    size_t size = v_BMPsize( PBMWidth(data), PBMHeight(data));
    CurrentPtr->data = new char _FAR_ [size];
    memcpy( CurrentPtr->data, data, size );
    strncpy( CurrentPtr->name, name, MAXFILE+MAXEXT);
    return CurrentIdx;
}

char _FAR_ * _pascal_ zSmallView::GetSprite( int which )
{
    zSprite *temp = FindSprite( which );
    if ( temp )
        return temp->data;
    return 0;
}


char _FAR_ * _pascal_ zSmallView::GetSprite( char *name )
{
    zSprite *temp = FindSprite( name );
    if ( temp )
        return temp->data;
    return 0;
}

char _FAR_ * _pascal_ zSmallView::GetSprite( void )
{
    if ( CurrentPtr )
        return CurrentPtr->data;
    else
        return 0;
}

zSprite * _pascal_ zSmallView::FindSprite( int which, int SetCur )
{
    if ( CurrentIdx == which && CurrentPtr ) return CurrentPtr;
    int Idx = 0;
    zSprite *TPtr = 0, *CPtr = head;
    while( 1 ) {
        if ( Idx == which )
             break;
        if ( CPtr->next == 0 )
            return 0;
        TPtr = CPtr;
        CPtr = CPtr->next;
        Idx ++;
    }
    if ( SetCur ) {
        CurrentIdx = Idx;
        TrailPtr = TPtr;
        CurrentPtr = CPtr;
    }
    return CPtr;
}

zSprite * _pascal_ zSmallView::FindSprite( char *name, int SetCur )
{
    if ( CurrentPtr && strcmpi( name, CurrentPtr->name) == 0 )
        return CurrentPtr;
    int Idx = 0;
    zSprite *TPtr = 0, *CPtr = head;
    while( 1 ) {
        if (strcmpi( name, CPtr->name ) == 0 )
            break;
        if ( CPtr->next == 0 )
            return 0;
        TPtr = CPtr;
        CPtr = CPtr->next;
        Idx ++;
    }
    if ( SetCur ) {
        CurrentIdx = Idx;
        TrailPtr = TPtr;
        CurrentPtr = CPtr;
    }
    return CPtr;
}

int _pascal_ zSmallView::FindSprite( zSprite *ptr, int SetCur )
{
    if ( CurrentPtr == ptr )
                return CurrentIdx;
    int Idx = 0;
    zSprite *TPtr = 0, *CPtr = head;
    if ( ptr == 0 ) {
        return 0;
    }
    if ( Count ) {
        while( CPtr != ptr ) {
            TPtr = CPtr;
            if ( !CPtr->next )
                return -1;
            CPtr = CPtr->next;
            Idx ++;
        }
    } else
        return -1;
    if ( SetCur ) {
        CurrentIdx = Idx;
        TrailPtr = TPtr;
        CurrentPtr = CPtr;
    }
    return Idx;
}

//
// Object ID Name: GET_SMALL_SPR_ID
//
//
// Object ID Name: PUT_SMALL_SPR_ID
//
DeclareObjH( GetPutSmallSpr )
{
    ShowImage(msg);

    Button::CAction( msg, 0 );

    int SprIdx;

    if ( msg->mouButton == LCLICK ) {
        SprIdx = LeftViewIndex;
    } else if ( msg->mouButton == RCLICK ) {
        SprIdx = RightViewIndex;
    } else
        return 0;

    ScrObj &t = *(ScrObj*)msg->ToObj;

    switch( t.CGetID()) {
        case GET_SMALL_SPR_ID:
            zSprite *t = SmallView.CurrentPtr;
            if( t && t->data ) {
                strncpy( SprView[SprIdx]->Name, t->name, MAXFILE+MAXEXT );
                SprView[SprIdx]->myCanvas->SaveUndo();
                SprView[SprIdx]->SetNewBuf( t->data );
            }
            break;
        case PUT_SMALL_SPR_ID:
            if ( PBMWidth( SprView[SprIdx]->data)) {
                SmallView.PutSprite( SprView[SprIdx]->data,
                                    SprView[SprIdx]->Name );
                SmallView.RedrawSMViewers();
            }
            break;
        defautl:
            break;
    }

    return 0;
}

void _pascal_ zSmallView::DrawSprite( ScrObj *t, zSprite *st )
{
    m_HideMouse;
    t->CDrawOutLine();
    if ( st ) {
        if ( st->data ) {
            x_pbm_to_bm( st->data, DataConvert );
            BytesPerLine = 16;
            ScreenPtr = CanvasData + 2;
            SCALE1( 0, 0, 16, 16, LBMWidth(DataConvert),
                    LBMHeight(DataConvert), DataConvert + 2);
            LBMHeight( CanvasData ) = (BYTE)16;
            LBMWidth( CanvasData ) = (BYTE)16;
            x_bm_to_pbm( CanvasData, DataConvert );
            v_Putbmp( t->x1()+1, t->y1()+1, DataConvert );
        } else
            t->CClrContext();
    } else
        t->CClrContext(FaceColor);
}

void _pascal_ zSmallView::RedrawSMViewers( void )
{
    int i = 0;
    WORD ViewerId = Viewer1ID;
    zSprite *temp = FindSprite( FirstViewerIdx );
    m_HideMouse;
    while( i < ViewerCount ) {
        DrawSprite( (ScrObj*)mmgr.CGetObject(ViewerId), temp );
        if ( temp )
            temp = temp->next;
        ViewerId ++;
        i ++;
    }
    if ( !CurrentViewer || CurrentIdx >= FirstViewerIdx + ViewerCount ) {
        CurrentViewer = (ScrObj*)mmgr.CGetObject( Viewer1ID );
    } else {
        if ( FirstViewerIdx > CurrentIdx )
            FirstViewerIdx = CurrentIdx;
        CurrentViewer = (ScrObj*)mmgr.CGetObject( CurrentIdx -
                                FirstViewerIdx + Viewer1ID );
    }
    CurrentViewer->CDrawOutLine(v_cWhite);
}

void _pascal_ zSmallView::DeleteAll( void )
{
    CurrentPtr = head;
    while( CurrentPtr ) {
        TrailPtr = CurrentPtr;
        CurrentPtr = CurrentPtr->next;
        delete TrailPtr;
    }
    NewList(0);
}


void _pascal_ zSmallView::DrawSpr( int x, int y )
{
    if ( !AniPtr )
        if ( !head )
            return;
        else
            AniPtr = head;
    if ( AniPtr->data ) {
        v_Putbmp( x, y, AniPtr->data );
        AniPtr = AniPtr->next;
    }
}

void _pascal_ zSmallView::DrawMaskedSpr( int x, int y )
{
    if ( !AniPtr )
        if ( !head )
            return;
        else
            AniPtr = head;
    if ( AniPtr->data ) {
        v_PutMaskedbmp( x, y, AniPtr->data);
        AniPtr = AniPtr->next;
    }
}

void _pascal_ PrintMsg( int d, int i )
{
    sprintf( buffer, "[+,-] %dms [1-9] %d H=25 E=100 [Q]", d, i );
    VideoPage = VisiblePageOffs;
    v_ShowMsgLen( 0, 225, MSG_LEN, TextColor, FaceColor, buffer );
    VideoPage = HiddenPageOffs;
    v_ShowMsgLen( 0, 225, MSG_LEN, TextColor, FaceColor, buffer );
}

void _pascal_ zSmallView::DoAnimation( int x, int y, int masked )
{
    if ( Count < 2 ) return;  // No point if only 1 sprite or none
    char _FAR_ *BackGrnd = 0;
    int MaxW = 0, MaxH = 0;
    AniPtr = head;
    while( AniPtr ) {
        if ( AniPtr->data ) {
            if ( PBMWidth( AniPtr->data ) > MaxW )
                MaxW = PBMWidth( AniPtr->data );
            if ( PBMHeight( AniPtr->data ) > MaxH )
                MaxH = PBMHeight( AniPtr->data );
        }
        AniPtr = AniPtr->next;
    }
    if ( masked ) {
        v_RectFill( x, y, x+MaxW, y+MaxH, v_cZero );
        BackGrnd = new char _FAR_ [MaxW*MaxH+2];
        v_Getbmp( x, y, MaxW, MaxH, BackGrnd );
    }
    int k, done = 0;
    static int Delay = 50, inc = 1;
    PrintMsg( Delay, inc );
    while( kbhit()) getch();
    while( !done ) {
        if ( kbhit()) {
            k = getch();
            if ( !k ) k = getch();
            switch( k ) {
                case '=':
                case '+':
                    if ( Delay < 950 )
                        Delay += inc;
                    break;
                case '-':
                    if ( Delay >= inc )
                        Delay -= inc;
                    break;
                case 27:
                case 'Q':
                case 'q':
                    done = 1;
                    break;
                case 'h':
                case 'H':
                    Delay = 25;
                    break;
                case 'e':
                case 'E':
                    Delay = 100;
                    break;
                default:
                    if ( k >= '1' && k <= '9' ) {
                        inc = k - '0';
                    }
                    break;
            }
            PrintMsg( Delay, inc );
        }
        if (!AniPtr ) AniPtr = head;
        if( AniPtr->data ) {
            if ( masked ) {
                v_Putbmp( x, y, BackGrnd );
                v_PutMaskedbmp( x, y, AniPtr->data );
            } else {
                v_Putbmp( x, y, AniPtr->data );
            }
            x_page_flip( 0, 0 );
            VideoPage = HiddenPageOffs;
            delay( Delay );
        }
        AniPtr = AniPtr->next;
    }
    while( kbhit()) getch();
    delete BackGrnd;
}

#define SPRLIB_SIG 0xFE0A
#define SPRHEADER_SIG  0xAECF
class zSprHeader
{
public:
    WORD Size;
    char Name[MAXFILE+MAXEXT];

    _pascal_ zSprHeader( char *name = 0, WORD size=0 );
    _pascal_ ~zSprHeader() {}
};

_pascal_ zSprHeader::zSprHeader( char *name, WORD size )
    :Size(size)
{
    if ( name )
        strncpy( Name, name, MAXFILE+MAXEXT );
}


int _pascal_ zSmallView::LoadLib( void )
{
    if ( !LibName ) return 0;
    int h = open( LibName, O_BINARY|O_RDONLY, S_IREAD );
    if ( h < 0 ) {
        printf("Failed to open file '%s'\n", LibName );
        perror( "open" );
//        v_PrintMsg( 0, 225, TextColor, FaceColor, "Open file failed.");
        return 0;
    }

    zSprHeader temp;
    int size, LoadCount;
    WORD sig;

    _read( h, &sig, sizeof(sig));
    if ( sig != SPRLIB_SIG ) { // Not right signature
//        v_PrintMsg( 0, 225, TextColor, FaceColor,
        printf("SprLib Signature not matched.\n" );
        return 0;
    }
    DeleteAll();
    _read( h, &LoadCount, sizeof( LoadCount ));

    zSprite *tmpSpr;
    char _FAR_ *buf;

    for( int i = 0; i < LoadCount; i ++ ) {
        size = _read(h, &sig, sizeof(sig));
        if ( sig != SPRHEADER_SIG || size == -1 ) {
            printf("header sig not matched.\n");
            return 0;
        }
        if ( size == 0 ) break;
        size = _read( h, &temp, sizeof(temp));
        if( size == -1 ) return 0;
        if( size == 0 ) break;
        buf = new char _FAR_ [temp.Size];
        size = _read( h, buf, temp.Size );
        if ( size == -1 ) return 0;
        tmpSpr = AddBlank();
        strncpy( tmpSpr->name, temp.Name, MAXFILE+MAXEXT );
        tmpSpr->data = buf;
        if ( size == 0 ) break;
    }
    if ( i != LoadCount ) {
        sprintf( buffer, "%d %d unmatched counts.", i, LoadCount );
        v_PrintMsgLen( 0, 225, MSG_LEN, TextColor, FaceColor, buffer );
    }
    close( h );
    return i;
}

int _pascal_ zSmallView::SaveLib( void )
{
    if ( !LibName ) return 0;
    int h = open( LibName, O_BINARY|O_RDWR|O_CREAT|O_TRUNC,
                  S_IREAD|S_IWRITE );
    if ( h < 0 ) return 0;

    zSprHeader temp;
    int size, sig, i = 0;

    sig = SPRLIB_SIG;
    _write( h, &sig, sizeof(sig));
    _write( h, &Count, sizeof(Count));

    sig = SPRHEADER_SIG;
    zSprite *tmpSpr = head;

    while( tmpSpr ) {
        if ( tmpSpr->data ) {
            _write( h, &sig, sizeof(sig));
            temp.Size = v_BMPsize( PBMWidth(tmpSpr->data),
                                PBMHeight(tmpSpr->data));
            strncpy( temp.Name, tmpSpr->name, MAXFILE+MAXEXT);
            _write( h, &temp, sizeof( temp ));
            size = write( h, tmpSpr->data, temp.Size );
            if ( size != temp.Size ) {
            }
            tmpSpr = tmpSpr->next;
            i ++;
        }
    }
    close(h);
    return i;
}

void _pascal_ zSmallView::SetLibName( char *n )
{
    if ( !n ) return;
    delete LibName;
    LibName = new char [strlen( n )+1];
    if ( LibName )
        strcpy( LibName, n );
}

void _pascal_ InitAnimate( void )
{
    m_HideMouse;
    SprView[1]->CClrContext();
    x_cp_vid_rect( 0, 0, v_Width, v_Height, 0, 0,
                   VisiblePageOffs, HiddenPageOffs, v_Width, v_Width );
    SmallView.DoAnimation( SprView[1]->x1()+1, SprView[1]->y1()+1, 1 );
    VideoPage = VisiblePageOffs;
    mmgr.CSendMsg( SprView[1], REDRAW );
}

//
// Object ID Name: SMALL_VIEW_1_ID
//
// Object ID Name: SMALL_VIEW_2_ID
//
// Object ID Name: SMALL_VIEW_3_ID
//
// Object ID Name: SMALL_VIEW_4_ID
//
// Object ID Name: SMALL_VIEW_5_ID
//
// Object ID Name: SMALL_VIEW_6_ID
//
// Object ID Name: SMALL_VIEW_7_ID
//
// Object ID Name: SMALL_VIEW_8_ID
//
// Object ID Name: SMALL_VIEW_9_ID
//
DeclareObjH( SmallViewers )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    switch( msg->actId ) {
        case REDRAW:
            SmallView.RedrawSMViewers();
            return 0;
        case MOUSE_OFF:
            m_HideMouse;
            CurrentViewer->CDrawOutLine(v_cWhite);
            return 0;
        default:
            break;
    }
    if ( msg->mouButton & CLICK ) {
        m_HideMouse;
        ScrObj *OldCur = CurrentViewer;
        WORD which = t.CGetID() - SmallView.Viewer1ID + FirstViewerIdx;
        if ( (int)which < SmallView.Count ) {
            CurrentViewer->CDrawOutLine();
            SmallView.FindSprite( which, 1 );
            t.CDrawOutLine( v_cWhite);
            CurrentViewer = &t;
        }
        if ( CurrentViewer == OldCur && msg->mouButton == RCLICK ) {

        }
    }
    return 0;
}

//
// Object ID Name: SMALV_SCROLLER_ID
//
DeclareObjH( SmalVScrol )
{
    ScrObj &t = *(ScrObj*)msg->ToObj;
    if ( msg->actId == REDRAW ) {
        t.CClrContext();
    }
    return 0;
}

//
// Object ID Name: SMALV_RIGHT_ID
//
// Object ID Name: SMALV_LEFT_ID
//
DeclareObjH( ScrollLR )
{
    ShowImage(msg);
    if ( !isClick( msg, 0 ) && msg->mouButton == LHOLD )
        if ( friction.peekTime() >= 1 )
            friction.start();
        else
            return 0;
    else if ( !(msg->mouButton & CLICK) )
        return 0;

    ScrObj &t = *(ScrObj*)msg->ToObj;

    switch( t.CGetID()) {
        case SMALV_RIGHT_ID:
            if ( SmallView.Count - FirstViewerIdx > SmallView.ViewerCount )
                 FirstViewerIdx += 3;
            if ( SmallView.CurrentIdx < FirstViewerIdx )
                SmallView.FindSprite( FirstViewerIdx, 1 );
            break;
        case SMALV_LEFT_ID:
            if ( FirstViewerIdx >= 3 )
                FirstViewerIdx -= 3;
            if (SmallView.CurrentIdx >=
                        (FirstViewerIdx+SmallView.ViewerCount))
                SmallView.FindSprite(FirstViewerIdx,1);
            break;
        default:
            return 0;
    }
    SmallView.RedrawSMViewers();
    return 0;
}

//
// Object ID Name: DELETE_ID
//
DeclareObjH( SmallVDelete )
{
    Button::CAction( msg, 0 );
    if ( msg->actId == YESNO_YES ) {
        SmallView.DeleteAll();
        SmallView.RedrawSMViewers();
    } else if ( msg->mouButton == LCLICK ) {
        if ( SmallView.RemoveCurrent() ) {
            SmallView.RedrawSMViewers();
        }
    } else if ( msg->mouButton == RCLICK ) {
        yesno( "Delete all Sprites?", (Object*)msg->ToObj );
    }
    return 0;
}

//
// Object ID Name: INSERT_ID
//
DeclareObjH( SmallVInsert )
{
    Button::CAction( msg, 0 );
    if ( msg->mouButton == LCLICK ) {
        SmallView.InsertBlankBefore();
    } else if ( msg->mouButton == RCLICK ) {
        SmallView.InsertBlankAfter();
    } else
        return 0;
    SmallView.RedrawSMViewers();
    return 0;
}


extern char * _pascal_ GetFileNameFromPath( char *path );

//
// Object ID Name: LOADSAVE_LIB_ID
//
DeclareObjH( LoadSaveLib )
{
    static int type = 0;

    Button::CAction( msg, 0 );
    switch( msg->actId ) {
        case FILE_DLG_REPLY:
            if ( type ) {
                getcwd( buffer, MAXPATH );
                PopDir(0);
                strcat( buffer, "\\" );
                strcat( buffer, ExportFileName );
                SmallView.SetLibName( buffer );
                if ( type == 1 ) {
                    FirstViewerIdx = 0;
                    CurrentViewer = 0;
                    SmallView.LoadLib();
                    SmallView.RedrawSMViewers();
                } else if ( type == 2 )
                    SmallView.SaveLib();
            }
            // No break; continue to next case.
        case FILE_DLG_CANCEL:
            type = 0;
            break;
        default:
            break;
    }
    if ( msg->mouButton & CLICK ) {
        PushDir();
        char *FileName;

        if ( SmallView.LibName ) {
            strcpy( buffer, SmallView.LibName );
            // Get the file name only from full path
            FileName = GetFileNameFromPath( buffer );
            strcpy( ExportFileName, FileName );
            // Cut the filename off within the buffer and change directory.
            FileName[0] = 0;
            Chdir( buffer );
        }
        if ( msg->mouButton == LCLICK ) {
            type = 1;
            strcpy( buffer, "Load ");
        } else {
            type = 2;
            strcpy( buffer, "Save ");
        }
        strcpy( buffer+5, "Sprite Library.");

        GetFileName( "*.slb", (Object*)msg->ToObj, GET_FILE_NAME, buffer );
    }
    return 0;
}
