#include <stdlib.h>
#include <dir.h>
#include <process.h>
#include <ctype.h>
#include <stdio.h>
#include "vidinter.h"  // Video interface
#include "timer.h"     // Interrupt driven timer
#include "fileutil.h"    // Read and write files
#include "depend.h"    // OS dependent stuff
#include "button.h"    // Implements Push buttons
#include "viewmgr.h"   // Implements View manager
#include "actids.h"	   // Action ids
#include "dialog.h"    // Simple Dialog Handler
#include "strbox.h"    // Implements the string box
#include "chkbox.h"    // Implements the Check Box
#include "keys.h"      // Key codes
#include "sprdesg.h"   // Header file for this module
#include "sprdesg.id"  // Object IDs for all the objects in this module
#include "paldlg.h"    // Header file for the Pallete Editor module
#include "filedlg.h"   // Header file for the File Dialog module
#include "buffer.h"    // Common buffer (1024 bytes)
#include "xbmtools.h"  // XLib bitmap tools
#include "scalebmp.h"  // Scaling linear bitmaps
#include "savesize.h"  // Pick part of a sprite for saving to file
#include "scale1.h"
#include "errcode.h"
#include "xfill.h"
#include "mousecur.h"   // Different mouse cursor shapes
#include "spcanvas.h"
#include "sprviewr.h"
#include "grpxprim.h"
#include "sprdesg.h"
//#include "xmouse.h"

#ifdef __MSDOS__
#include <dos.h>       // Needed for msdos
#include <alloc.h>     // for coreleft
#include <conio.h>     // getch and kbhit
#endif



zCanvas *Canvas[2];
int FillInterior;

//
// Object ID Name: CANVAS_1_ID
// sprite editor 1
//
// Object ID Name: CANVAS_2_ID
// sprite editor 2
//
//

int zCanvas::penType = PENCIL_ID;

void _pascal_ zCanvas::DrawGrids( BBOOL ColorZero )
{
    int i, x2, y2;

    x2 = sX2 - 1,
    y2 = sY2 - 1;
    m_HideMouse;
    switch( PixSize ) {
        case 1:
            CClrContext(BtnLowColor);
            break;
        case 2:
            for( i = OrigX; i < x2; ) {
                v_PutLine( i++, OrigY, i, y2, v_cZero );
                v_PutLine( i++, OrigY, i, y2, BtnLowColor );
            }
            for( i = OrigY; i < y2; i += 2 )
                v_PutLine( OrigX, i, x2, i, v_cZero );
            break;
        default:
            if ( ColorZero ) {
                v_RectFill( OrigX, OrigY, sX2, OrigY+PixSize, v_cZero );
                int Pix1 = HalfPixSize+OrigY;
                for( i = OrigX+HalfPixSize; i < x2; i += PixSize )
                    v_PutPixel( i, Pix1, FaceColor );
                v_Getbmp( OrigX, OrigY, sX2-sX1-1, PixSize, CanvasData );
                for( i = OrigY+PixSize; i < y2; i += PixSize )
                    v_Putbmp( OrigX, i, CanvasData );
                break;
            }
            for( i = OrigX; i < x2; i += PixSize )
                v_PutLine( i, OrigY, i, y2, BtnLowColor);
            for( i = OrigY; i < y2; i += PixSize )
                v_PutLine( OrigX, i, x2, i, BtnLowColor);
            break;
    }
    m_ShowMouse;
}


void _pascal_ zCanvas::RedrawCanvas( void )
{
    int w, h, ow, oh;

    if ( PBMByteWidth(Viewer->data)) {
        // Convert Sprite data from PBM to Linear Bitmap
        x_pbm_to_bm( Viewer->data, DataConvert );

        ow = LBMWidth(DataConvert);
        oh = LBMHeight(DataConvert);
        w = ow * PixSize;
        h = oh * PixSize;
        BytesPerLine = w;
        ScreenPtr = CanvasData + 2;
        SCALE1( 0, 0, w, h, ow, oh, DataConvert + 2 );
        LBMHeight( CanvasData ) = (BYTE)h;
        LBMWidth( CanvasData) = (BYTE)w;

        // and back to PBM!!! I need a direct PBM to PBM scaling function.
        x_bm_to_pbm( CanvasData, DataConvert );
    }

    m_HideMouse;
    x_cp_vid_rect( 0, 0, 320, 240, 0, 0,
                   VisiblePageOffs, HiddenPageOffs, 320, 320 );
    VideoPage = HiddenPageOffs;

    if ( Adjusted == TRUE) {
        Adjusted = FALSE;
        v_RectFill( sX1 - 1, sY1 - 1, sX2 + 2, sY2 + 2, FaceColor );

        w = PixSize * Viewer->Width;
        h = PixSize * Viewer->Height;

        sX2 = sX1 + w + 1;
        sY2 = sY1 + h + 1;

        CHighLiteOutLine();
        CRestoreOutLine();
        CDraw3DOutLine();
    }

    if ( PixSize < 3 )         DrawGrids();
    else                       DrawGrids(TRUE);
    if ( PBMByteWidth(Viewer->data))
        v_PutMaskedbmp( OrigX, OrigY, DataConvert );
    if ( PixSize > 2 )         DrawGrids();
    x_page_flip(0,0);
    VideoPage = VisiblePageOffs;

    m_ShowMouse;

} // zCanvas::RedrawCanvas


// Draws a pixel on the Canvas and the sprite viewer
// and Draws the grid dot if color is zero.
void _pascal_ zCanvas::PutPixel( void )
{
    m_HideMouse;
    v_PutPixel( sx, sy, color );
    switch( PixSize ) {
        case 1:
            if ( color == 0 )
                v_PutPixel( cx, cy, FaceColor );
            else
                v_PutPixel( cx, cy, color );
            break;
        case 2:
            v_RectFill( cx, cy, cx+PixSize, cy+PixSize, color );
            if ( color == 0 )
                v_PutPixel( cx + (PixSize>>1), cy + (PixSize>>1), BtnLowColor );
            break;
        default:
            int x = cx + 1, y = cy + 1;
            if ( color )
                v_RectFill( x, y, x+PixSize-1, y+PixSize-1, color );
            else {
                v_RectFill( x, y, x+PixSize-1, y+PixSize-1, v_cZero );
                v_PutPixel( x+HalfPixSize-1, y+HalfPixSize-1, FaceColor );
            }
            break;
    }
}

// Take a x, y coor on the viewer and translate that into a coor on canvas.
// Then draw both pixels on screen.
void _pascal_ zCanvas::PutPixelPos( int x, int y )
{
    sx = x, sy = y;
    cx = ((x-Viewer->OrigX)*PixSize)+OrigX;
    cy = ((y-Viewer->OrigY)*PixSize)+OrigY;
    PutPixel();
}

// Print the message of where mouse's position is.
void _pascal_ zCanvas::ShowMousePos( void )
{
    int x = objMouseX-OrigX, y = objMouseY-OrigY;
    sprintf(buffer, "x: %3d  y: %3d", x/PixSize+1, y/PixSize+1 );
    v_PrintMsgLen( 0, 225, MSG_LEN, TextColor, FaceColor, buffer );
}


// Set the drawing color for the Canvas
int _pascal_ zCanvas::SetColor( BYTE mouBut )
{
    color = mouBut & BUTTON;
    if ( color == 1 )       color = LeftColor;
    else if ( color == 2 )  color = RightColor;
    else {
        m_MouseWindow( 0, 0, v_Width - 1, v_Height - 4 );
        return (1);
    }
    return 0;
}


// First point/click, whatever, just need to do some housekeeping stuff
void _pascal_ zCanvas::InitPenDown( void )
{
    cx = cy = -1;
    penDown = TRUE;
    m_HideMouse;
    m_MouseWindow( OrigX, OrigY, sX2-1, sY2-2 );
    m_UpdateMouse;
    if ( objMouseX > sX2-1 ) m_MouseX = objMouseX = sX2 - 1;
    if ( objMouseY > sY2-2 ) m_MouseY = objMouseY = sY2 - 2;
    SaveUndo();
    mmgr.CSetHoldMsgCnt( FALSE );
}

// Convert the mouse x, y to grided pos for the canvas and find
// the position on the actual sprite(need to draw on viwer)
void _pascal_ zCanvas::TranslateXY( void )
{
    if ( RestrictY == FALSE ) {
        cx = objMouseX - OrigX;
        cx /= PixSize;
        sx = cx + Viewer->OrigX;
        cx = cx * PixSize + OrigX;
    }

    if ( RestrictX == FALSE ) {
        cy = objMouseY - OrigY;
        cy /= PixSize;
        sy = cy + Viewer->OrigY;
        cy = cy * PixSize + OrigY;
    }
}

// Finish doing Freehand draw(plotting random points), drawing a line, circle
// rectangle, and etc.
void _pascal_ zCanvas::CompleteDraw( void )
{
    m_HideMouse;
    penDown = FALSE;
    Viewer->CopyToBuffer();
    m_MouseWindow( 0, 0, v_Width - 1, v_Height - 4 );
    mmgr.CSetHoldMsgCnt( TRUE );
}

// Freehand draw(plotting random points)
DeclareObjHClass( zCanvas, DrawPixels )
{
    // Is this the first point
    if ( penDown == FALSE ) {
        // Set color
        if ( SetColor( msg->mouButton) ) {
            penDown = FALSE;
            return 0;
        }
        InitPenDown();
    }

    // mouse x, y are not very useful, so:
    TranslateXY();

    // Did the mouse move?
    if ( msg->mouButton & HOLD ) {
        // If so, draw a new pixel
        PutPixel();
        ShowMousePos();
        m_ShowMouse;
    } else if ( msg->mouButton & CLICK ) {
        // Click => down and up => done
        PutPixel();
        CompleteDraw();
    }
    return 0;
}


// Draw filled circle and rectangle
int _pascal_ zCanvas::FillIt( int Ptsx1, int Ptsy1 )
{
    if ( !FillInterior ) return 0;
    switch( penType ) {
        case CIRCLE_ID:
            int SaveColor = color, color1;
            if ( color < 256/2 ) {
                color++;
                color1 = color + 1;
            } else {
                color--;
                color1 = color - 1;
            }

            Viewer->CopyToScreen();

            VideoPage = HiddenPageOffs;

            Viewer->CClrContext();
            int rad1 = abs(sx-Ptsx1), rad2 = abs(sy-Ptsy1);
            if ( rad1 < rad2 ) rad1 = rad2;

            Viewer->OutLineSprite( color );
            CRLDev.CDrawCircle( Ptsx1, Ptsy1, rad1, Viewer, this );

            x_flood_fill( Ptsx1, Ptsy1, VideoPage, color1 );
            Viewer->CopyToBuffer();

            VideoPage = VisiblePageOffs;
            v_PutMaskedbmp( Viewer->OrigX, Viewer->OrigY, Viewer->data );
            Viewer->OutLineSprite( color );
            color = SaveColor;
            x_flood_fill( Ptsx1, Ptsy1, VideoPage, color );
            CRLDev.CDrawCircle( Ptsx1, Ptsy1, rad1, Viewer, this );
            Viewer->OutLineSprite( v_cWhite );
            break;
        case RECT_ID:
            Viewer->CopyToScreen();
            v_RectFill( min(Ptsx1, sx), min(Ptsy1, sy),
                        max(Ptsx1, sx)+1, max(Ptsy1, sy)+1, color );
            break;
        default:
            return 0;
    }
    CompleteDraw();
    mmgr.CSendMsg( this, REDRAW );
    return 1;
}

// Draw stuff that requires two points: like a line, a circle
DeclareObjHClass( zCanvas, Draw2 )
{
    static int Ptx1, Pty1, Ptsx1, Ptsy1;
    static int vx2, vy2;

    // First point?
    if ( penDown == FALSE ) {
        vx2 = Viewer->x2()-1, vy2 = Viewer->y2()-1;
        // Set color
        if ( SetColor( msg->mouButton) ) {
            penDown = FALSE;
            return 0;
        }
        InitPenDown();
        TranslateXY();
        Ptx1 = cx, Pty1 = cy;
        Ptsx1 = sx, Ptsy1 = sy;
        m_HideMouse;
        // Copy Sprite data to buffer to save any changes made
        Viewer->CopyToBuffer();
        CopyToBuffer( CanvasData );
        // Setup up double buffer for smooth redraws.
        x_cp_vid_rect( 0, 0, 320, 240, 0, 0,
                       VisiblePageOffs, HiddenPageOffs, 320, 320 );
        m_ShowMouse;
        return 0;
    }
    TranslateXY();

    // Mouse move and button still down?
    if ( msg->mouButton & HOLD ) {
        m_HideMouse;
        // Let's do doublebuffer, so draw on hidden page!
        VideoPage = HiddenPageOffs;
        // draw line on old sprite
        Viewer->CopyToScreen();
        CopyToScreen( CanvasData );
        switch( penType ) {
            case RECT_ID:
                // Need this canvas for drawing
                CRLDev.CDrawRect( Ptsx1, Ptsy1, sx, sy, this );
                break;
            case LINE_ID:
                CRLDev.CDrawLine( Ptsx1, Ptsy1, sx, sy, this );
                break;
            case CIRCLE_ID:
                int rad1 = abs(sx-Ptsx1), rad2 = abs(sy-Ptsy1);
                if ( rad1 < rad2 ) rad1 = rad2;
                // Need viewer for clipping
                CRLDev.CDrawCircle( Ptsx1, Ptsy1, rad1, Viewer, this );
                break;
            default:
                break;
        }
        ShowMousePos();
        x_page_flip( 0, 0 );
        m_ShowMouse;
    } else if ( msg->mouButton & CLICK ) {
        m_HideMouse;
        if ( !FillIt( Ptsx1, Ptsy1 ))
            CompleteDraw();
    }
    return 0;
}


// do the flood fill
DeclareObjHClass( zCanvas, Fill )
{
    if ( msg->mouButton & CLICK ) {
        InitPenDown();
        SetColor( msg->mouButton );
        TranslateXY();
        m_HideMouse;
        int BoundColor;
        if ( color < 256/2)
            BoundColor = color + 1;
        else
            BoundColor = color - 1;
        Viewer->OutLineSprite( BoundColor );
        x_flood_fill( sx, sy, VideoPage, color );
        Viewer->OutLineSprite();
        CompleteDraw();
        mmgr.CSendMsg( this, REDRAW );
    }
    return 0;
}

DeclareObjHClass( zCanvas, SwapSprColor )
{
    if ( msg->mouButton & CLICK ) {
        InitPenDown();
        SetColor( msg->mouButton );
        TranslateXY();
        m_HideMouse;
        int RColor = v_GetPixel( sx, sy );
        SwapColor( (BYTE _FAR_*)Viewer->data, RColor, color );
        Viewer->CopyToScreen();
        CompleteDraw();
        mmgr.CSendMsg( this, REDRAW );
    }
    return 0;
}

void _pascal_ zCanvas::CheckKey( int key )
{
    static BBOOL SpeedChanged = FALSE;

    switch( toupper( key )) {
        case 'F':
            FillInterior = !FillInterior;
            ChangeCursor();
            break;
        case 'V':
            RestrictX = FALSE;
            RestrictY = !RestrictY;
            break;
        case 'H':
            RestrictY = FALSE;
            RestrictX = !RestrictX;
            break;
        default:
            if ( SpeedChanged )
                MouseHVSpeed();
            else
                MouseHVSpeed( 25, 25 );
            SpeedChanged = !SpeedChanged;
            break;
    }
}


void _pascal_ zCanvas::CopyToBuffer( char _FAR_ *buf )
{
    v_Getbmp( OrigX, OrigY, 128, 128, buf );
}

void _pascal_ zCanvas::CopyToScreen( char _FAR_ *buf )
{
    v_Putbmp( OrigX, OrigY, buf );
}

void _pascal_ zCanvas::AdjustPixSize( void )
{
    int w = max( Viewer->Width, Viewer->Height );

    PixSize = 128 / w;
    Viewer->PixSize = PixSize;
    HalfPixSize = PixSize / 2;

    Adjusted = TRUE;
}


void _pascal_ zCanvas::SaveUndo( void )
{
    char _FAR_ *SprPtr = Viewer->data;
    char _FAR_ *UndoPtr = UndoData;

    if (PBMByteWidth(SprPtr)) {
        m_HideMouse;
        v_Getbmp( Viewer->OrigX, Viewer->OrigY,
                  Viewer->Width, Viewer->Height,
                  UndoPtr );
    } else
        PBMByteWidth(UndoPtr) = 0;
}


void _pascal_ zCanvas::DoUndo( void )
{
    if ( PBMByteWidth(UndoData)) {
        Viewer->SetNewBuf(UndoData);
    } else
        PBMByteWidth(Viewer->data) = 0;
}

// Change the mouse pointer's shape
void _pascal_ zCanvas::ChangeCursor( void )
{
    char _FAR_ *CurPtr;
    switch( penType ) {
        case PENCIL_ID:
            CurPtr = PencilCur;
            break;
        case LINE_ID:
            CurPtr = LineCur;
            break;
        case CIRCLE_ID:
            if ( FillInterior )
                CurPtr = FillCircleCur;
            else
                CurPtr = CircleCur;
            break;
        case RECT_ID:
            if ( FillInterior )
                CurPtr = FillRectCur;
            else
                CurPtr = RectCur;
            break;
        case FILL_ID:
            CurPtr = FillCur;
            break;
        case SWAP_COLOR_ID:
            CurPtr = SwapCur;
            break;
        default:
            return;
    }
    m_HideMouse;
    x_define_mouse_cursor( CurPtr, MouseCursorColor );
    m_UpdateMouse;
}


DeclareObjHClass( zCanvas, DoDraw )
{
    switch( penType ) {
        case PENCIL_ID:
            return DrawPixels( msg );
        case RECT_ID:
        case LINE_ID:
        case CIRCLE_ID:
            return Draw2( msg );
        case FILL_ID:
            return Fill( msg );
        case SWAP_COLOR_ID:
            return SwapSprColor( msg );
        default:
            break;
    }
    return 0;
}



// Finally! the actual handler for the canvases!
// This is the last function for doing editing on the canvas.

DeclareObjH( CanvasEdit )
{
    static BBOOL CursorChanged = FALSE;
    zCanvas &t = *(zCanvas*)(msg->ToObj);

    // Process Action ID
    switch ( msg->actId ) {
    case 0:
    case CANCEL_CLICK:  // mouse click cancel
        // Check which one to draw.
        return t.DoDraw( msg );
    case REDRAW:
        t.RedrawCanvas();
        break;
    case MOUSE_ON:      // Mouse is inside the canvas and has just moved.
        if ( !CursorChanged ) {
	    t.ChangeCursor();
	    CursorChanged = TRUE;
	}
        t.ShowMousePos();
        break;
    case MOUSE_OFF:     // Mouse moved off the canvas
        if ( CursorChanged ) {
//andout	    x_define_def_mouse_cursor( MouseCursorColor );
	   x_define_mouse_cursor( defCursor, MouseCursorColor );
	    CursorChanged = FALSE;
        }
        break;
    case KEY_DOWN:      // A key is press while mouse is on the canvas
        t.CheckKey( msg->keyDown );
	break;
    default:
	break;
    }
    return 0;
}
