#include <dos.h>
#include <stdlib.h>
#include <signal.h>
#include <conio.h>
#include <stdio.h>
#include "depend.h"
#include "macros.h"
#include "timer.h"

#if USE_TIMER

#ifndef NormalTimer
    unsigned ticks = 1;
    unsigned newDivisor = 1;
    unsigned Counter = 11;
#endif


//
// Timer class Member functions.
// Written by Joel Chen 1994-1995
// For WDG Library by Joel Chen
//

//
// Start
//

unsigned _pascal_ Timer::start( void )
{
#ifdef __MSDOS__
    #ifndef NormalTimer
        startTime = ticks;
    #else
        static unsigned curTime;
        asm {
            mov AH, 0;
            int 1AH;
            mov curTime, DX;
        }
        startTime = curTime;
    #endif
#endif

    return startTime;
}

//
// Stop
//

unsigned _pascal_ Timer::stop( void )
{
    static unsigned endTime;
#ifdef __MSDOS__
    #ifndef NormalTimer
        endTime = ticks;
    #else
        asm {
            mov AH, 0;
            int 1AH;
            mov endTime, DX;
        }
    #endif
#endif

    elapsedTime = endTime - startTime;
    return elapsedTime;
}

//
// Continue
//

unsigned _pascal_ Timer::Continue( void )
{
#ifdef __MSDOS__
    #ifndef NormalTimer
        startTime = ticks - elapsedTime;
    #else
        static unsigned curTime;
        asm {
            mov AH, 0;
            int 1AH;
            mov curTime, DX;
        }
        startTime = curTime - elapsedTime;
    #endif
#endif

    return startTime;
}

//
// peekTime
//

unsigned _pascal_ Timer::peekTime( void )
{
#ifdef __MSDOS__
    #ifndef NormalTimer
        return (ticks - startTime);
    #else
        static unsigned endTime;
        asm {
            mov AH, 0;
            int 1AH;
            mov endTime, DX;
        }
        return (endTime-startTime);
    #endif
#endif

}


//
// RunDown
//

void _pascal_ Timer::RunDown( unsigned t )
{
#ifdef __MSDOS__
    #ifndef NormalTimer
        while((ticks-startTime) < t);
    #else
        static unsigned endTime;
        do {
            asm {
                mov AH, 0;
                int 1AH;
                mov endTime, DX;
            }
        } while( (endTime - startTime) < t);
    #endif
#endif

}

//
// Timer interrupt handling
//

#ifndef NormalTimer

DeclareOldIsr( oldTimerIsr );
DeclareNewIsr( newTimerIsr )
{
    Counter --;
    // New speed counter is up
    if ( Counter == 0 ) {
        oldTimerIsr();
        Counter = 11;
    }
    ticks ++;
    asm {
        mov AL, 20H
        out 20H, AL
    }
}

//
// Change speed of timer
//
#define TIMER_ISR 0x08
int _pascal_ Timer::ChangeSpeed( unsigned newDiv, unsigned newSpeed )
{
    static int IsrInstalled = 0;
    if ( (newDiv == 0 && IsrInstalled == 0 ) )
        return 0;

    if ( !IsrInstalled ) {
        IsrInstalled = 1;
        installisr( oldTimerIsr, newTimerIsr, TIMER_ISR);
    }
    if ( newDiv != 1 ) { // 0 => restore    greater than 1 => change speed
        //
        // The following code is from "Programming Solutions Handbook
        // for IBM Microcomputers" by Julio Sanchez and Maria P. Canton
        //
        asm {
            CLI         // Interrupts off while resetting
            MOV AL,00110110B    // 00xx xxxx = channel 0
                        // xx11 xxxx = write LSB then MSM
                        // xxxx 011x = mode 3
                        // xxxx xxx0 = binary system
            OUT 43H,AL
            MOV BX,newSpeed // New divisor
            MOV AL,BL
            OUT 40H,AL      // Send LSB
            MOV AL,BH
            OUT 40H,AL      // Send MSB
            STI
        }
    }
    if ( !newDiv ) {
        IsrInstalled = 0;
        restoreisr( TIMER_ISR, oldTimerIsr ); // remove interrupt handler
    } else
        newDivisor = newDiv;
    return 1;
}
#endif // routines required for fast timer


// #define TEST_TIMER
#ifdef TEST_TIMER

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
void main ( void )
{
#ifdef NormalTimer
	#error Required fast timer to be turned on.
#endif
    if (!Timer::CInstallTimer(10, 6555)) {
		printf("Required fast timer.\n");  // secondary protections. :)
        exit(-1);
    }
    Timer w;
    int i = 0;
    for (i = 500; i <= 10000; i += 500 ) {
		w.start();
		delay( i );
		w.stop();
		printf("delay: %4d ms  time: %3d\n", i, w.time() );
	}
	Timer::CRemoveTimer();
}

#endif

#endif // USE_TIMER
