#include <stdio.h>
#include "manager.h"
#include "mouinter.h"
#include "object.h"
#include "depend.h"
#include "actids.h"

#ifdef __MSDOS__
#include <dos.h>
#endif

#ifdef KBHIT
#include <conio.h>
#endif

//////////////////////
//                  //
// ObjMgr functions //
//                  //
//////////////////////

//
// Initialize static member of ObjMgr
//
IntCList ObjMgr::PublicIntCs;

/////////////////////////////////////////////////////
//                                                 //
// Private Function                                //
// This function does not deallocate the object    //
// it only remove the object from the list         //
//                                                 //
/////////////////////////////////////////////////////
BBOOL _pascal_
AtomList::UnRegObj (Atom *obj )
{
    Atom *tmpObjPtr, *prevObjPtr;

    tmpObjPtr = ObjHead;
    prevObjPtr = 0;
    
    if (obj == 0 ) return FALSE;       // Make sure it's not a dummy pointer
    if (obj == Current ) Current = 0;  // Can't directly unreg current
                                       // because need to ensure the object
                                       // before current is not lost.
    while (tmpObjPtr ) {
        if (tmpObjPtr == obj ) {       // found
            if (!tmpObjPtr->next )     // Tail or only one object

                if (!prevObjPtr )       // check if only one
                    ObjTail = ObjHead = 0;  // only one => all gone
                else {
                    prevObjPtr->next = 0;      // it's tail! take it out.
                    ObjTail = prevObjPtr;
                }

            else if (prevObjPtr )              // in Middle?
                prevObjPtr->next = tmpObjPtr->next;   // take it out
            else                               // else it's head
                ObjHead = (Object*)tmpObjPtr->next;     // take out head

            TotalObj--;
            return TRUE;
        }
		prevObjPtr = tmpObjPtr;
        tmpObjPtr = tmpObjPtr->next;
	}
	return FALSE;
} // end of private function UnRegObj

//
// Constructors
//
_pascal_ AtomList::AtomList( Atom *atom )
{
    if ( atom )
        TotalObj = 1;
    else
        TotalObj = 0;
    ObjTail = ObjHead = Current = atom;
}

ObjList& _pascal_ ObjList::operator= ( const ObjList &y )
{
    if ( this != &y ) {
        (*this).TotalObj = y.TotalObj;
        (*this).ObjHead = y.ObjHead;
        (*this).ObjTail = y.ObjTail;
        (*this).Current = y.Current;
    }
    return *this;
}

ObjMgr& _pascal_ ObjMgr::operator= (const ObjMgr &y)
{
    if (this != &y ) {
		(*this).stop = y.stop;
//        (*this).InterCList = y.InterCList;
//        (*this).PrivateInterCList = y.InterCList;
		ObjList::operator=( y );
    }
    return *this;
}

Object* _pascal_
ObjList::CGetObject (WORD id )
{
    Object *tmpObjPtr;
    tmpObjPtr = (Object*)ObjHead;
    
    while (tmpObjPtr ) { 
        if (tmpObjPtr->ObjID == id )   // found
            return tmpObjPtr;
        tmpObjPtr =(Object*) tmpObjPtr->next;
    }
    return 0;
}


BBOOL _pascal_
AtomList::CRegObject (Atom *obj )
{
    Atom *tmpPtr;
    tmpPtr = ObjHead;

    while ( tmpPtr ) {
        if ( tmpPtr == obj )
            return FALSE;
        tmpPtr = tmpPtr->next;
    }

    if (!ObjHead )
        ObjHead = ObjTail = obj;
    else { 
        ObjTail->next = obj;
        ObjTail = obj;
    }
    obj->next = 0;

    TotalObj ++;
    return TRUE;
}


//////////////////////////////////////////////////////////////
//                                                          //
// CUnRegObjByPtr( Object* object )                         //
// Unregister an object by passing in its pointer           //
//                                                          //
//////////////////////////////////////////////////////////////
BBOOL _pascal_
AtomList::CUnRegObjByPtr (Atom *obj )
{
    return UnRegObj( obj );
}

//////////////////////////////////////////////////////////////
//                                                          //
// CUnRegObjByID( WORD id )                                 //
// Unregister an object by passing in its id                //
//                                                          //
//////////////////////////////////////////////////////////////
BBOOL _pascal_
ObjList::CUnRegObjByID (WORD id )
{
    return UnRegObj( CGetObject(id) );
}

//////////////////////////////////////////////////////////////
//                                                          //
// CRegInt( INTC regInt  )                                  //
//                                                          //
//////////////////////////////////////////////////////////////
void _pascal_
IntCList::CRegInt ( INTC regInt, Object *ObjInt )
{
    InterCs *temp = new InterCs( regInt, ObjInt );
    if ( temp )
        if ( CRegObject( temp ) == FALSE )
            delete temp;
}

BBOOL _pascal_
IntCList::CUnRegInt ( INTC unRegInt )
{
    InterCs *temp = (InterCs*)ObjHead;

    while( temp ) {
        if ( temp->InterC == unRegInt ) {
            return UnRegObj( (Atom*)temp );
        }
        temp = (InterCs*)temp->CGetNext();
    }
    return FALSE;
}

void _pascal_
ObjMgr::CRegInt ( INTC regInt, Object *ObjInt, BBOOL Private )
{
    if ( Private ) {
        PrivateIntCs.CRegInt( regInt, ObjInt );
    } else
        PublicIntCs.CRegInt( regInt, ObjInt );
}

BBOOL _pascal_
ObjMgr::CUnRegInt ( INTC unRegInt, BBOOL Private )
{
    if ( Private )
        return PrivateIntCs.CUnRegInt( unRegInt );
    else
        return PublicIntCs.CUnRegInt( unRegInt );
}

int _pascal_ IntCList::CallIntCs( void )
{
    static InterCs *BackUpPtr;

    InterCs *headPtr = (InterCs*)ObjHead;

    while( headPtr ) {
        BackUpPtr = headPtr;
        headPtr = (InterCs*)headPtr->CGetNext();

        switch( BackUpPtr->CallBack() ) {
            case UNREGME:
                CUnRegInt (BackUpPtr->InterC );
                break;
            case UNREGMEnSTOP:
                CUnRegInt (BackUpPtr->InterC );
                CurrentMgr->CStopLoop();
                break;
            case STOPNOW:
                CurrentMgr->CStopLoop();
                return 0;
            default:
                break;
        }
	}
    return 1;
}

int _pascal_
ObjMgr::CMgrLoop ( void )
{
    ObjMgr   *oldMgr;   // Since we are going to take over,
                        // better save the old guy

    stop = 0;
    oldMgr = CurrentMgr;
    CurrentMgr = this;

    while ( !stop ) {
    	#ifdef __LINUX__
    	mouse_update();
    	#endif

        if ( PublicIntCs.CallIntCs() )
            PrivateIntCs.CallIntCs();
        ObjHandler ();
    }
    CurrentMgr = oldMgr;
    return 0;
}

/////////////////////////////
// End of ObjMgr functions //
/////////////////////////////
